@@ -195,6 +195,8 @@ macro_rules! cached_method {
 macro_rules! cached_lambda {
     {
         $(#[key($key_expr:expr, $key_ty:ty)])?
+        $( #[__use_try($is_try:tt)] )?
+        $( #[__cache_ty($cache_ty:ty)] )?
         $vis:vis fn $name:ident ($( $arg_name:ident : $arg_ty:ty ),* $(,)?) $( -> $ret_ty:ty )?
         { $($tt:tt)* }
     } => {
@@ -219,7 +221,9 @@ macro_rules! cached_lambda {
             }
 
             macro_rules! __make_ret_type_impl {
-                (->  $sub_ret_ty:ty) => {
+                ($sub_cache_ty:ty) => { $sub_cache_ty };
+                ($sub_cache_ty:ty; $sub_ret_ty:ty) => { $sub_cache_ty };
+                (;  $sub_ret_ty:ty) => {
                     $sub_ret_ty
                 };
                 () => {
@@ -233,7 +237,8 @@ macro_rules! cached_lambda {
             // We can't avoid boxing because lambdas can't refer to themselves.
             // Hopefully inlined out?
             type [<$name:camel KeyType>] = __make_key_type_impl!($({$key_ty})* ($($arg_ty,)*));
-            type [<$name:camel RetType>] = __make_ret_type_impl!($(-> $ret_ty)*);
+            type [<$name:camel CacheType>] = __make_ret_type_impl!($($cache_ty)* $(; $ret_ty)*);
+            type [<$name:camel RetType>] = __make_ret_type_impl!($(; $ret_ty)*);
 
             // TODO: it's possible to fix the lifetimes here so we can use references more easily
             // (but doesn't seem important for now...)
@@ -242,13 +247,14 @@ macro_rules! cached_lambda {
             );
 
             struct [<$name:camel FuncHolder>] {
-                cache : cached::UnboundCache<[<$name:camel KeyType>], [<$name:camel RetType>]>,
+                cache : cached::UnboundCache<[<$name:camel KeyType>], [<$name:camel CacheType>]>,
             }
 
             impl [<$name:camel FuncHolder>] {
                 #[apply($crate::cached_method)]
                 #[self_id(self_)]
                 #[key(__make_key_expr_impl!($({$key_expr})* ($($arg_name.clone(),)*)))]
+                $(#[__use_try($is_try)])*
                 #[cache_expr(cache)]
                 fn call(&mut self, func_arg : &[<$name:camel OtherFuncHolder>],
                     $( $arg_name : $arg_ty,)*) $(-> $ret_ty)* {
@@ -273,5 +279,20 @@ macro_rules! cached_lambda {
             #[allow(unused_mut)]
             let mut $name = |$($arg_name,)*| holder.call(&other_holder, $($arg_name,)*);
         }
-    }
+    };
+
+    (
+        $(#[key($key_expr:expr, $key_ty:ty)])?
+        #[use_try]
+        $vis:vis fn $name:ident ($( $arg_name:ident : $arg_ty:ty ),* $(,)?) -> Result<$ret_ty:ty>
+        { $($tt:tt)* }
+    ) => {
+        cached_lambda! {
+            $(#[key($key_expr, $key_ty)])*
+            #[__use_try(true)]
+            #[__cache_ty($ret_ty)]
+            $vis fn $name ($( $arg_name : $arg_ty,)*) -> Result<$ret_ty>
+            { $($tt)* }
+        }
+    };
 }
