@@ -1,15 +1,13 @@
-use crate::{all_imports::OpSize, circuit::EinsumAxes, util::arc_ref_clone};
-use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
 use std::{
     collections::hash_map::Entry::{self, Vacant},
     iter::zip,
     sync::Arc,
 };
 
+use anyhow::Result;
 use bit_iter::BitIter;
 use itertools::Itertools;
-
-use crate::{all_imports::RInts, pyo3_prelude::*, sv, util::BitMask128};
+use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
 
 use super::{
     circuit_optimizer::OptimizationContext, computational_node::EinsumArgs, prelude::*,
@@ -17,17 +15,19 @@ use super::{
     Rearrange, ScalarConstant, Scatter, Symbol,
 };
 use crate::{
+    all_imports::{OpSize, RInts},
+    circuit::EinsumAxes,
     filter_by_variant,
     opt_einsum::optimize_einsum_spec_cached,
+    pyo3_prelude::*,
     rearrange_spec::{shape_to_op_shape, RearrangeSpec},
-    tensor_util::{
-        compose, Shape, Slice, TensorAxisIndex, TensorIndex, TensorInvariantError, USlice,
-    },
+    sv,
+    tensor_util::{compose, Shape, Slice, TensorAxisIndex, TensorIndex, USlice},
     union_find::UnionFind,
     unwrap,
     util::{
-        filter_out_idx, filter_to_idx, intersection_all, inverse_permutation, is_unique,
-        unique_to_appearance, AxisInt,
+        arc_ref_clone, filter_out_idx, filter_to_idx, intersection_all, inverse_permutation,
+        is_unique, unique_to_appearance, AxisInt, BitMask128,
     },
 };
 
@@ -402,10 +402,7 @@ pub fn expand_maybe_diag(
     result
 }
 
-pub fn make_broadcast(
-    node: &CircuitRc,
-    out_shape: &Shape,
-) -> Result<CircuitRc, TensorInvariantError> {
+pub fn make_broadcast(node: &CircuitRc, out_shape: &Shape) -> Result<CircuitRc> {
     let rank_dif = out_shape.len() - node.info().rank();
     let input_ints = node
         .info()
@@ -885,9 +882,7 @@ pub fn distribute_all(einsum: &Einsum) -> Option<Add> {
     if add_op_idxs.len() > 1 {
         distributed = distributed
             .map_children(&mut |node: CircuitRc| match &**node {
-                Circuit::Einsum(inner) => {
-                    Ok::<CircuitRc, CircuitConstructionError>(distribute_all(inner).unwrap().rc())
-                }
+                Circuit::Einsum(inner) => Ok(distribute_all(inner).unwrap().rc()),
                 _ => {
                     panic!();
                 }
