@@ -1,25 +1,34 @@
-use super::circuit_optimizer::OptimizationContext;
-use super::Einsum;
+use std::collections::BTreeMap;
+
+use anyhow::{bail, Result};
+use macro_rules_attribute::apply;
+use pyo3::exceptions::PyValueError;
+use thiserror::Error;
+
 use super::{
-    named_axes::get_axis_leaves, prelude::*, Index, ModuleNodeArgSpec, ModuleNodeSpec, Symbol,
+    circuit_optimizer::OptimizationContext, named_axes::get_axis_leaves, prelude::*, Einsum, Index,
+    ModuleNodeArgSpec, ModuleNodeSpec, Symbol,
+};
+use crate::{
+    all_imports::TensorIndex,
+    circuit::{
+        algebraic_rewrite::einsum_nest_optimize,
+        circuit_optimizer::OptimizationSettings,
+        deep_map_op, deep_map_unwrap,
+        named_axes::{axis_of_name, set_named_axes},
+        visit_circuit_unwrap, visit_circuit_with_parents, Concat, EinsumAxes, ModuleNode,
+    },
+    pyo3_prelude::*,
+    python_error_exception,
+    set_cover::set_cover,
+    tensor_util::TensorAxisIndex,
 };
 
-use crate::circuit::algebraic_rewrite::einsum_nest_optimize;
-use crate::circuit::circuit_optimizer::OptimizationSettings;
-use crate::circuit::named_axes::axis_of_name;
-use crate::circuit::{
-    deep_map_op, visit_circuit_unwrap, visit_circuit_with_parents, Concat, EinsumAxes,
-};
-use crate::circuit::{deep_map_unwrap, named_axes::set_named_axes, CircResult, ModuleNode};
-use crate::set_cover::set_cover;
-use crate::tensor_util::TensorAxisIndex;
-use crate::{all_imports::TensorIndex, pyo3_prelude::*};
-use std::collections::BTreeMap;
 #[pyfunction]
-pub fn batch_to_concat(circuit: CircuitRc, axis: usize, num_batches: usize) -> CircResult {
+pub fn batch_to_concat(circuit: CircuitRc, axis: usize, num_batches: usize) -> Result<CircuitRc> {
     let l = circuit.info().shape[axis];
     if l % num_batches != 0 {
-        return Err(CircuitConstructionError::BatchNumberDoesntDivide { l, num_batches });
+        bail!(BatchError::NumberDoesntDivide { l, num_batches });
     }
     let batch_size = l / num_batches;
     let circuit = deep_map_unwrap(circuit, |x| {
@@ -73,11 +82,11 @@ pub fn batch_to_concat(circuit: CircuitRc, axis: usize, num_batches: usize) -> C
     //     is.0.compiler_print();
     // }
     if input_specs.is_empty() {
-        return Err(CircuitConstructionError::BatchAxisOriginatesTooHigh {});
+        bail!(BatchError::AxisOriginatesTooHigh {});
     }
     let module_spec =
         ModuleNodeSpec::new_extract(circuit.clone(), input_specs.clone(), None, false)
-            .ok_or(CircuitConstructionError::BatchAxisOriginatesTooHigh {})?;
+            .ok_or(BatchError::AxisOriginatesTooHigh {})?;
     // println!("module spec");
     // module_spec.spec_circuit.compiler_print();
     let module_spec = module_spec.resize(
@@ -149,21 +158,36 @@ pub fn batch_to_concat(circuit: CircuitRc, axis: usize, num_batches: usize) -> C
         circuit.compiler_print();
         println!("new");
         result.compiler_print();
-        return Err(CircuitConstructionError::BugError {});
+        panic!();
     }
     Ok(result)
 }
 
+#[apply(python_error_exception)]
+#[base_error_name(Batch)]
+#[base_exception(PyValueError)]
+#[derive(Error, Debug, Clone)]
+pub enum BatchError {
+    #[error("num_batches {num_batches} doesn't divide length {l}")]
+    NumberDoesntDivide { l: usize, num_batches: usize },
+
+    #[error("Would need to batch multiple axes, only supports one")]
+    RequiresMultipleAxes {},
+
+    #[error("Batching axis originates too high")]
+    AxisOriginatesTooHigh {},
+}
+
 #[pyfunction]
 #[pyo3(name = "batch_einsum")]
-pub fn batch_einsum_py(einsum: Einsum, settings: OptimizationSettings) -> CircResult {
+pub fn batch_einsum_py(einsum: Einsum, settings: OptimizationSettings) -> Result<CircuitRc> {
     batch_einsum(
         &einsum,
         &mut OptimizationContext::new_settings_circuit(settings, einsum.crc()),
     )
 }
 
-pub fn batch_einsum(einsum: &Einsum, context: &mut OptimizationContext) -> CircResult {
+pub fn batch_einsum(einsum: &Einsum, context: &mut OptimizationContext) -> Result<CircuitRc> {
     let max_single_mem = context.cache.max_single_tensor_numel;
     assert!(einsum.info().numel_usize() <= max_single_mem);
 
@@ -216,7 +240,7 @@ pub fn batch_einsum(einsum: &Einsum, context: &mut OptimizationContext) -> CircR
     }
     let ints_needed = set_cover(&covers);
     if ints_needed.len() != 1 {
-        return Err(CircuitConstructionError::BatchingRequiresMultipleAxes {});
+        bail!(BatchError::RequiresMultipleAxes {});
     }
 
     let batch_int = ints_needed[0];
