@@ -1,21 +1,19 @@
+use anyhow::Result;
+use macro_rules_attribute::apply;
+use pyo3::{pyfunction, PyObject, Python};
+use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
+
 use super::{
     deep_map_op_context_preorder_stoppable, deep_map_preorder, deep_map_preorder_unwrap,
     prelude::*, visit_circuit, HashBytes,
 };
-use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
-
 use crate::{pycall, pycallable};
-use macro_rules_attribute::apply;
-use pyo3::{pyfunction, PyObject, Python};
 
 #[apply(pycallable)]
 #[pyo3(name = "filter_nodes")]
-pub fn filter_nodes<F>(
-    circuit: CircuitRc,
-    filter: F,
-) -> Result<HashSet<CircuitRc>, CircuitConstructionError>
+pub fn filter_nodes<F>(circuit: CircuitRc, filter: F) -> Result<HashSet<CircuitRc>>
 where
-    F: Fn((circuit, CircuitRc)) -> Result<bool, _>,
+    F: Fn((circuit, CircuitRc)) -> Result<bool>,
 {
     let mut result: HashSet<CircuitRc> = HashSet::default();
     visit_circuit(circuit, |circuit: CircuitRc| {
@@ -58,11 +56,11 @@ pub fn update_nodes_py(
     circuit: CircuitRc,
     matcher: PyObject,
     updater: PyObject,
-) -> Result<CircuitRc, CircuitConstructionError> {
+) -> Result<CircuitRc> {
     let nodes = filter_nodes_py(circuit.clone(), matcher)?;
     deep_map_preorder(circuit, |x| {
         if nodes.contains(&x) {
-            pycall!(updater, (x,), CircuitConstructionError)
+            pycall!(updater, (x,), anyhow)
         } else {
             Ok(x)
         }
@@ -84,22 +82,18 @@ pub fn path_get(circuit: CircuitRc, path: CircuitPath) -> Option<CircuitRc> {
     Some(cur)
 }
 
-pub fn update_path<F>(
-    circuit: CircuitRc,
-    path: &CircuitPath,
-    updater: F,
-) -> Result<CircuitRc, CircuitConstructionError>
+pub fn update_path<F>(circuit: CircuitRc, path: &CircuitPath, updater: F) -> Result<CircuitRc>
 where
-    F: Fn(CircuitRc) -> Result<CircuitRc, CircuitConstructionError>,
+    F: Fn(CircuitRc) -> Result<CircuitRc>,
 {
     fn recurse<F>(
         circuit: CircuitRc,
         path: &CircuitPath,
         path_idx: usize,
         updater: &F,
-    ) -> Result<CircuitRc, CircuitConstructionError>
+    ) -> Result<CircuitRc>
     where
-        F: Fn(CircuitRc) -> Result<CircuitRc, CircuitConstructionError>,
+        F: Fn(CircuitRc) -> Result<CircuitRc>,
     {
         if path_idx == path.len() {
             return updater(circuit);
@@ -123,8 +117,6 @@ pub fn update_path_py(
     circuit: CircuitRc,
     path: CircuitPath,
     updater: PyObject,
-) -> Result<CircuitRc, CircuitConstructionError> {
-    update_path(circuit, &path, |x| {
-        pycall!(updater, (x.clone(),), CircuitConstructionError)
-    })
+) -> Result<CircuitRc> {
+    update_path(circuit, &path, |x| pycall!(updater, (x.clone(),), anyhow))
 }
