@@ -1,30 +1,33 @@
-use crate::{
-    all_imports::Shape,
-    circuit::{print::repr_circuit_deep_compiler, repr::RustRepr},
-};
-use rustc_hash::FxHashMap as HashMap;
 use std::io::prelude::*;
 
+use anyhow::{bail, Result};
+use num_bigint::BigUint;
+use rustc_hash::FxHashMap as HashMap;
+
 use super::{
     canonicalize::deep_canonicalize,
     circuit_utils::has_leaf_symbols,
     compiler_heuristics::deep_maybe_distribute,
     compiler_strip::strip_names,
     deep_map_op_context,
-    deep_rewrite::deep_heuristic_nest_adds,
-    deep_rewrite::{compiler_simp, deep_optimize_einsums, deep_push_down_index},
+    deep_rewrite::{
+        compiler_simp, deep_heuristic_nest_adds, deep_optimize_einsums, deep_push_down_index,
+    },
     flat_concat, get_compatible_dtype,
     scheduled_execution::{
         circuit_to_schedule, circuit_to_schedule_naive_toposort, Schedule, SchedulingError,
     },
     CircuitNode, CircuitRc, HashBytes, ModuleNode, ModuleNodeSpec,
 };
-use crate::circuit::CircResult;
-use crate::opt_einsum::EinsumSpec;
-use crate::pyo3_prelude::*;
-use crate::python_println;
-use crate::{py_types::Tensor, timed, util::apply_fn_until_same};
-use num_bigint::BigUint;
+use crate::{
+    all_imports::Shape,
+    circuit::{print::repr_circuit_deep_compiler, repr::RustRepr},
+    opt_einsum::EinsumSpec,
+    py_types::Tensor,
+    pyo3_prelude::*,
+    python_println, timed,
+    util::apply_fn_until_same,
+};
 
 // a lot of the function boundries in this are there because it would be too hard to optimize things jointly
 // not because we don't need to optimize things jointly
@@ -179,8 +182,8 @@ pub struct OptimizationCache {
     pub canonicalized: HashMap<HashBytes, Option<CircuitRc>>,
     pub module_specs_optimized_same_settings: HashMap<ModuleNodeSpec, ModuleNodeSpec>,
     pub module_specs_scheduled_same_settings: HashMap<ModuleNodeSpec, Schedule>,
-    pub module_specs_expanded_shape: HashMap<(ModuleNodeSpec, Vec<Shape>), CircResult>,
-    pub module_specs_expanded: HashMap<(ModuleNodeSpec, Vec<HashBytes>), CircResult>,
+    pub module_specs_expanded_shape: HashMap<(ModuleNodeSpec, Vec<Shape>), CircuitRc>,
+    pub module_specs_expanded: HashMap<(ModuleNodeSpec, Vec<HashBytes>), CircuitRc>,
     pub stripped_names: HashMap<HashBytes, Option<CircuitRc>>,
     pub times_distributed: usize,
     pub simplification_log: Vec<&'static str>,
@@ -309,7 +312,7 @@ pub fn optimize_circuit(circuit: CircuitRc, context: &mut OptimizationContext) -
 pub fn optimize_to_schedule(
     circuit: CircuitRc,
     settings: OptimizationSettings,
-) -> Result<Schedule, SchedulingError> {
+) -> Result<Schedule> {
     let context = &mut OptimizationContext::new_settings_circuit(settings, circuit.clone());
     let verbose = settings.verbose;
     if verbose > 0 {
@@ -336,12 +339,9 @@ pub fn optimize_to_schedule(
 }
 
 #[pyfunction]
-pub fn optimize_and_evaluate(
-    circuit: CircuitRc,
-    settings: OptimizationSettings,
-) -> Result<Tensor, SchedulingError> {
+pub fn optimize_and_evaluate(circuit: CircuitRc, settings: OptimizationSettings) -> Result<Tensor> {
     if has_leaf_symbols(circuit.clone(), &Default::default()) {
-        return Err(SchedulingError::LeafSymbols {});
+        bail!(SchedulingError::LeafSymbols {});
     }
     let schedule = optimize_to_schedule(circuit, settings)?;
 
@@ -358,7 +358,7 @@ pub fn optimize_and_evaluate(
 pub fn optimize_and_evaluate_many(
     circuits: Vec<CircuitRc>,
     settings: OptimizationSettings,
-) -> Result<Vec<Tensor>, SchedulingError> {
+) -> Result<Vec<Tensor>> {
     let schedule = optimize_to_schedule_many(circuits, settings)?;
     Ok(schedule.evaluate_many(settings))
 }
@@ -367,7 +367,7 @@ pub fn optimize_and_evaluate_many(
 pub fn optimize_to_schedule_many(
     circuits: Vec<CircuitRc>,
     settings: OptimizationSettings,
-) -> Result<Schedule, SchedulingError> {
+) -> Result<Schedule> {
     let flat_concatted = flat_concat(circuits.clone()).rc();
     let mut schedule = optimize_to_schedule(flat_concatted, settings)?;
     schedule.split_shapes = Some(
