@@ -1,20 +1,17 @@
-use std::{collections::BTreeSet, iter::zip};
+use std::{collections::BTreeSet, hash::Hash, iter::zip};
 
-use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
-
-use crate::{
-    opt_einsum::EinsumSpec,
-    pyo3_prelude::*,
-    tensor_util::{TensorAxisIndex, TensorIndex},
-};
 use num_bigint::BigUint;
+use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
 
 use super::{
     apply_fn_cache, deep_map_op_pre_new_children, deep_map_unwrap, visit_circuit_unwrap,
     ArrayConstant, Circuit, CircuitNode, CircuitRc, HashBytes, Index,
 };
-use crate::tensor_util::TorchDeviceDtype;
-use std::hash::Hash;
+use crate::{
+    opt_einsum::EinsumSpec,
+    pyo3_prelude::*,
+    tensor_util::{TensorAxisIndex, TensorIndex, TorchDeviceDtype},
+};
 
 #[pyfunction]
 pub fn cast_circuit(circ: CircuitRc, device_dtype: &TorchDeviceDtype) -> CircuitRc {
