@@ -1,6 +1,6 @@
-use super::circuit_optimizer::OptimizationContext;
-use super::{deep_map_op_context, Circuit, CircuitNode, CircuitRc};
-
+use super::{
+    circuit_optimizer::OptimizationContext, deep_map_op_context, Circuit, CircuitNode, CircuitRc,
+};
 use crate::pyo3_prelude::*;
 
 /// don't change symbols bc their names matter for correctness
