@@ -1,36 +1,36 @@
-use crate::py_types::{
-    einops_repeat, einsum_py, make_diagonal_py, scalar_to_tensor, scalar_to_tensor_py, PY_UTILS,
-};
-use crate::pyo3_prelude::*;
-use crate::sv;
-use crate::util::cumsum;
+use std::{cmp::Ordering, iter::zip};
+
+use anyhow::{Context, Result};
 use itertools::Itertools;
 use macro_rules_attribute::apply;
 use num_bigint::BigUint;
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
 use smallvec::SmallVec as Sv;
-use std::cmp::Ordering;
-use std::iter::zip;
 
 use super::{
     algebraic_rewrite::{add_deduplicate, add_flatten_once, remove_add_few_input},
     prelude::*,
     py_circuit_items::PY_CIRCUIT_ITEMS,
-    CachedCircuitInfo, CircuitConstructionError, PyCircuitBase, ScalarConstant, Shape,
+    CachedCircuitInfo, ConstructError, PyCircuitBase, ScalarConstant, Shape,
 };
 use crate::{
     circuit_node_auto_impl,
     circuit_node_extra_impl,
     new_rc_unwrap,
     opt_einsum::EinsumSpec,
-    py_types::{ExtraPySelfOps, PyEinsumAxes, Tensor},
+    py_types::{
+        einops_repeat, einsum_py, make_diagonal_py, scalar_to_tensor, scalar_to_tensor_py,
+        ExtraPySelfOps, PyEinsumAxes, Tensor, PY_UTILS,
+    },
+    pyo3_prelude::*,
     rearrange_spec::RearrangeSpec, // TODO: use new rearrange spec
+    sv,
     tensor_util::{
-        broadcast_shapes, Slice, TensorAxisIndex, TensorIndex, TensorInvariantError,
-        TorchDeviceDtype, TorchDeviceDtypeOp,
+        broadcast_shapes, Slice, TensorAxisIndex, TensorIndex, TorchDeviceDtype, TorchDeviceDtypeOp,
+    },
+    util::{
+        cumsum, dict_to_list, {hashmap_collect_except_duplicates, AxisInt},
     },
-    util::dict_to_list,
-    util::{hashmap_collect_except_duplicates, AxisInt},
 };
 
 // we could use u64 instead of u8 if we wanted.
@@ -93,20 +93,17 @@ impl CircuitNode for Einsum {
         Box::new(self.args.iter().map(|a| a.0.clone()))
     }
 
-    fn map_children_enumerate<F, E>(&self, mut f: F) -> Result<Self, CircuitConstructionError>
+    fn map_children_enumerate<F>(&self, mut f: F) -> Result<Self>
     where
-        CircuitConstructionError: From<E>,
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc, E>,
+        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
     {
         Self::try_new(
             self.args
                 .iter()
                 .enumerate()
-                .map(
-                    move |(i, (circ, axes))| -> Result<_, CircuitConstructionError> {
-                        Ok((f(i, circ.clone())?, axes.clone()))
-                    },
-                )
+                .map(move |(i, (circ, axes))| -> Result<_> {
+                    Ok((f(i, circ.clone())?, axes.clone()))
+                })
                 .collect::<Result<_, _>>()?,
             self.out_axes.clone(),
             self.name.clone(),
@@ -117,11 +114,7 @@ impl CircuitNode for Einsum {
         self.get_spec().flops()
     }
 
-    fn eval_tensors(
-        &self,
-        tensors: &[Tensor],
-        device_dtype: &TorchDeviceDtype,
-    ) -> Result<Tensor, super::TensorEvalError> {
+    fn eval_tensors(&self, tensors: &[Tensor], device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
         Python::with_gil(|py| {
             if self.args.is_empty() {
                 return Ok(scalar_to_tensor_py(py, 1., sv![], device_dtype.clone())?);
@@ -157,25 +150,27 @@ impl Einsum {
         Box::new(self.args.iter().map(|(_, axes)| axes))
     }
 
-    pub fn shape_map(&self) -> Result<HashMap<AxisInt, usize>, CircuitConstructionError> {
+    pub fn shape_map(&self) -> Result<HashMap<AxisInt, usize>> {
         let mut out = HashMap::default();
         for (circ, axes) in &self.args {
             if circ.info().rank() != axes.len() {
-                return Err(CircuitConstructionError::EinsumLenShapeDifferentFromAxes {
+                return Err(ConstructError::EinsumLenShapeDifferentFromAxes {
                     circuit_name: circ.name_cloned(),
                     circuit_len_shape: circ.info().rank(),
                     len_axes: axes.len(),
-                });
+                }
+                .into());
             }
             for (&circuit_shape, axis) in circ.info().shape.iter().zip(axes) {
                 let existing_shape = *out.entry(*axis).or_insert(circuit_shape);
                 if existing_shape != circuit_shape {
-                    return Err(CircuitConstructionError::EinsumShapeDifferent {
+                    return Err(ConstructError::EinsumShapeDifferent {
                         circuit_name: circ.name_cloned(),
                         circuit_shape,
                         axis: *axis as usize,
                         existing_shape,
-                    });
+                    }
+                    .into());
                 }
             }
         }
@@ -183,11 +178,7 @@ impl Einsum {
         Ok(out)
     }
     #[apply(new_rc_unwrap)]
-    pub fn try_new(
-        args: EinsumArgs,
-        out_axes: EinsumAxes,
-        name: Option<String>,
-    ) -> (Result<Self, CircuitConstructionError>) {
+    pub fn try_new(args: EinsumArgs, out_axes: EinsumAxes, name: Option<String>) -> Result<Self> {
         // TODO: reduce axes nums!
         let mut out = Self {
             args,
@@ -201,7 +192,7 @@ impl Einsum {
 
         for a in &out.out_axes {
             if !map.contains_key(a) {
-                return Err(CircuitConstructionError::EinsumOutputNotSubset {});
+                return Err(ConstructError::EinsumOutputNotSubset {}.into());
             }
         }
 
@@ -213,7 +204,7 @@ impl Einsum {
         spec: &EinsumSpec,
         circuits: &[CircuitRc],
         name: Option<String>,
-    ) -> Result<Self, CircuitConstructionError> {
+    ) -> Result<Self> {
         // TODO: maybe check int sizes!
         assert_eq!(spec.input_ints.len(), circuits.len());
         let to_idx =
@@ -303,7 +294,7 @@ impl Einsum {
         string: &str,
         nodes: Vec<CircuitRc>,
         name: Option<String>,
-    ) -> Result<Self, CircuitConstructionError> {
+    ) -> Result<Self> {
         let (input_ints, output_ints) = EinsumSpec::string_to_ints(string.to_owned());
         Self::try_new(zip(nodes, input_ints).collect(), output_ints, name)
     }
@@ -314,7 +305,7 @@ impl Einsum {
         string: &str,
         nodes: Vec<CircuitRc>,
         name: Option<String>,
-    ) -> Result<Self, CircuitConstructionError> {
+    ) -> Result<Self> {
         let (input_ints, output_ints) = EinsumSpec::fancy_string_to_ints(string.to_owned());
         Self::try_new(zip(nodes, input_ints).collect(), output_ints, name)
     }
@@ -337,7 +328,7 @@ impl Einsum {
         spec: EinsumSpec,
         nodes: Vec<CircuitRc>,
         name: Option<String>,
-    ) -> Result<Self, CircuitConstructionError> {
+    ) -> Result<Self> {
         Self::try_from_spec(&spec, &nodes, name)
     }
 
@@ -405,12 +396,9 @@ impl Einsum {
     }
     #[staticmethod]
     #[args(nodes = "*")]
-    pub fn elementwise_broadcasted(
-        nodes: Vec<CircuitRc>,
-        name: Option<String>,
-    ) -> Result<Einsum, CircuitConstructionError> {
+    pub fn elementwise_broadcasted(nodes: Vec<CircuitRc>, name: Option<String>) -> Result<Einsum> {
         let out_shape = broadcast_shapes(&nodes.iter().map(|x| x.info().shape.clone()).collect())
-            .map_err(|e| CircuitConstructionError::TensorInvariantError { err: e })?;
+            .context("failed to broadcast for einsum mul")?;
         let mut prev_one_shape = out_shape.len().saturating_sub(1) as u8;
         Einsum::try_new(
             nodes
@@ -517,10 +505,7 @@ pub struct Add {
 
 impl Add {
     #[apply(new_rc_unwrap)]
-    pub fn try_new(
-        nodes: Vec<CircuitRc>,
-        name: Option<String>,
-    ) -> (Result<Self, CircuitConstructionError>) {
+    pub fn try_new(nodes: Vec<CircuitRc>, name: Option<String>) -> Result<Self> {
         // TODO: reduce axes nums!
         let mut out = Self {
             nodes,
@@ -528,16 +513,15 @@ impl Add {
             info: Default::default(),
         };
         out.name = out.auto_name(name);
-        if let Err(TensorInvariantError::NotBroadcastable { shapes }) = out.compute_shape_maybe() {
-            return Err(CircuitConstructionError::SumNotBroadcastable { shapes });
-        }
+        out.compute_shape_maybe()
+            .context("Sum isn't broadcastable")?;
         out.init_info()
     }
 
     pub fn try_from_counts(
         nodes: &HashMap<CircuitRc, usize>,
         name: Option<String>,
-    ) -> Result<Self, CircuitConstructionError> {
+    ) -> Result<Self> {
         Self::try_new(
             nodes
                 .iter()
@@ -547,9 +531,8 @@ impl Add {
         )
     }
 
-    fn compute_shape_maybe(&self) -> Result<Shape, TensorInvariantError> {
-        let shape = broadcast_shapes(&self.nodes.iter().map(|x| x.info().shape.clone()).collect());
-        shape
+    fn compute_shape_maybe(&self) -> Result<Shape> {
+        broadcast_shapes(&self.nodes.iter().map(|x| x.info().shape.clone()).collect())
     }
 }
 
@@ -640,10 +623,9 @@ impl CircuitNode for Add {
         Box::new(self.nodes.iter().cloned())
     }
 
-    fn map_children_enumerate<F, E>(&self, mut f: F) -> Result<Self, CircuitConstructionError>
+    fn map_children_enumerate<F>(&self, mut f: F) -> Result<Self>
     where
-        CircuitConstructionError: From<E>,
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc, E>,
+        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
     {
         Self::try_new(
             self.nodes
@@ -659,11 +641,7 @@ impl CircuitNode for Add {
         self.info.numel() * self.nodes.len()
     }
 
-    fn eval_tensors(
-        &self,
-        tensors: &[Tensor],
-        device_dtype: &TorchDeviceDtype,
-    ) -> Result<Tensor, super::TensorEvalError> {
+    fn eval_tensors(&self, tensors: &[Tensor], device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
         if tensors.is_empty() {
             return Ok(scalar_to_tensor(0.0, sv!(), device_dtype.clone())?);
         }
@@ -716,11 +694,7 @@ pub struct Rearrange {
 impl Rearrange {
     #[apply(new_rc_unwrap)]
 
-    pub fn try_new(
-        node: CircuitRc,
-        spec: RearrangeSpec,
-        name: Option<String>,
-    ) -> (Result<Self, CircuitConstructionError>) {
+    pub fn try_new(node: CircuitRc, spec: RearrangeSpec, name: Option<String>) -> Result<Self> {
         // TODO: reduce axes nums!
         let mut out = Self {
             node: node.clone(),
@@ -730,7 +704,7 @@ impl Rearrange {
         };
         out.name = out.auto_name(name);
         if !out.spec.is_valid() {
-            return Err(CircuitConstructionError::InvalidRearrangeSpec { spec: out.spec });
+            return Err(ConstructError::InvalidRearrangeSpec { spec: out.spec }.into());
         }
         // check that known sizes of each axis divide the operand shape
         let input_sizes: Vec<u64> = out
@@ -758,10 +732,11 @@ impl Rearrange {
             .any(|(i, l)| *l as u64 % input_sizes[i] != 0);
 
         if input_sizes.len() != node.info().rank() || any_sizes_dont_divide {
-            return Err(CircuitConstructionError::RearrangeWrongInputShape {
+            return Err(ConstructError::RearrangeWrongInputShape {
                 spec: out.spec,
                 shape: node.info().shape.clone(),
-            });
+            }
+            .into());
         }
         out.init_info()
     }
@@ -827,10 +802,9 @@ impl CircuitNode for Rearrange {
         Box::new(std::iter::once(self.node.clone()))
     }
 
-    fn map_children_enumerate<F, E>(&self, mut f: F) -> Result<Self, CircuitConstructionError>
+    fn map_children_enumerate<F>(&self, mut f: F) -> Result<Self>
     where
-        CircuitConstructionError: From<E>,
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc, E>,
+        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
     {
         Self::try_new(
             f(0, self.node.clone())?,
@@ -839,16 +813,12 @@ impl CircuitNode for Rearrange {
         )
     }
 
-    fn eval_tensors(
-        &self,
-        tensors: &[Tensor],
-        _device_dtype: &TorchDeviceDtype,
-    ) -> Result<Tensor, super::TensorEvalError> {
-        Ok(einops_repeat(
+    fn eval_tensors(&self, tensors: &[Tensor], _device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
+        einops_repeat(
             &tensors[0],
             self.spec.to_einops_string(false),
             self.spec.letter_sizes(),
-        )?)
+        )
     }
 
     fn intermediate_cost_bound(&self) -> usize {
@@ -896,15 +866,11 @@ impl Rearrange {
     }
 
     #[staticmethod]
-    pub fn from_einops_string(
-        node: CircuitRc,
-        string: &str,
-        name: Option<String>,
-    ) -> Result<Self, CircuitConstructionError> {
+    pub fn from_einops_string(node: CircuitRc, string: &str, name: Option<String>) -> Result<Self> {
         Rearrange::try_new(
             node,
             RearrangeSpec::from_einops_string(string).ok_or(
-                CircuitConstructionError::RearrangeStringInvalid {
+                ConstructError::RearrangeStringInvalid {
                     string: string.to_owned(),
                 },
             )?,
@@ -912,7 +878,7 @@ impl Rearrange {
         )
     }
 
-    pub fn conform_to_input_shape(&self, coerce: bool) -> Result<Self, CircuitConstructionError> {
+    pub fn conform_to_input_shape(&self, coerce: bool) -> Result<Self> {
         Ok(Rearrange::try_new(
             self.node.clone(),
             self.spec
@@ -942,19 +908,16 @@ pub struct Index {
 
 impl Index {
     #[apply(new_rc_unwrap)]
-    pub fn try_new(
-        node: CircuitRc,
-        index: TensorIndex,
-        name: Option<String>,
-    ) -> (Result<Self, CircuitConstructionError>) {
+    pub fn try_new(node: CircuitRc, index: TensorIndex, name: Option<String>) -> Result<Self> {
         let mut index = index;
         let node_rank = node.info().rank();
         match index.0.len().cmp(&node_rank) {
             Ordering::Greater => {
-                return Err(CircuitConstructionError::IndexRankTooHigh {
+                return Err(ConstructError::IndexRankTooHigh {
                     index_rank: index.0.len(),
                     node_rank,
-                });
+                }
+                .into());
             }
             Ordering::Less => index.0.extend(vec![
                 TensorAxisIndex::Slice(Slice {
@@ -1027,10 +990,9 @@ impl CircuitNode for Index {
         Box::new(std::iter::once(self.node.clone()))
     }
 
-    fn map_children_enumerate<F, E>(&self, mut f: F) -> Result<Self, CircuitConstructionError>
+    fn map_children_enumerate<F>(&self, mut f: F) -> Result<Self>
     where
-        CircuitConstructionError: From<E>,
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc, E>,
+        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
     {
         Self::try_new(
             f(0, self.node.clone())?,
@@ -1039,11 +1001,7 @@ impl CircuitNode for Index {
         )
     }
 
-    fn eval_tensors(
-        &self,
-        tensors: &[Tensor],
-        _device_dtype: &TorchDeviceDtype,
-    ) -> Result<Tensor, super::TensorEvalError> {
+    fn eval_tensors(&self, tensors: &[Tensor], _device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
         Python::with_gil(|py| {
             PY_CIRCUIT_ITEMS
                 .circ_compiler_util
@@ -1144,21 +1102,18 @@ pub struct Concat {
 
 impl Concat {
     #[apply(new_rc_unwrap)]
-    pub fn try_new(
-        nodes: Vec<CircuitRc>,
-        axis: usize,
-        name: Option<String>,
-    ) -> (Result<Self, CircuitConstructionError>) {
+    pub fn try_new(nodes: Vec<CircuitRc>, axis: usize, name: Option<String>) -> Result<Self> {
         if nodes.is_empty() {
-            return Err(CircuitConstructionError::ConcatZeroNodes {});
+            return Err(ConstructError::ConcatZeroNodes {}.into());
         }
         let node_shape = &nodes[0].info().shape;
         for node in nodes.iter() {
             for (i, s) in node.info().shape.iter().enumerate() {
                 if i != axis && *s != node_shape[i] {
-                    return Err(CircuitConstructionError::ConcatShapeDifferent {
+                    return Err(ConstructError::ConcatShapeDifferent {
                         shapes: nodes.iter().map(|x| x.info().shape.clone()).collect(),
-                    });
+                    }
+                    .into());
                 }
             }
         }
@@ -1198,10 +1153,9 @@ impl CircuitNode for Concat {
         Box::new(self.nodes.iter().cloned())
     }
 
-    fn map_children_enumerate<F, E>(&self, mut f: F) -> Result<Self, CircuitConstructionError>
+    fn map_children_enumerate<F>(&self, mut f: F) -> Result<Self>
     where
-        CircuitConstructionError: From<E>,
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc, E>,
+        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
     {
         Self::try_new(
             self.nodes
@@ -1226,11 +1180,7 @@ impl CircuitNode for Concat {
         ]
     }
 
-    fn eval_tensors(
-        &self,
-        tensors: &[Tensor],
-        _device_dtype: &TorchDeviceDtype,
-    ) -> Result<Tensor, super::TensorEvalError> {
+    fn eval_tensors(&self, tensors: &[Tensor], _device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
         Python::with_gil(|py| {
             PY_UTILS
                 .torch
@@ -1324,17 +1274,18 @@ impl Scatter {
         index: TensorIndex,
         shape: Shape,
         name: Option<String>,
-    ) -> (Result<Self, CircuitConstructionError>) {
+    ) -> Result<Self> {
         let index_shape = index.apply_to_shape(&shape);
         if index.all_uslices().is_none() {
-            return Err(CircuitConstructionError::ScatterUnimplemented { index });
+            return Err(ConstructError::ScatterIndexTypeUnimplemented { index }.into());
         }
         if index_shape[..] != node.info().shape[..] {
-            return Err(CircuitConstructionError::ScatterShapeWrong {
+            return Err(ConstructError::ScatterShapeWrong {
                 shape,
                 index,
                 index_shape,
-            });
+            }
+            .into());
         }
 
         let mut out = Self {
@@ -1390,10 +1341,9 @@ impl CircuitNode for Scatter {
             .collect()]
     }
 
-    fn map_children_enumerate<F, E>(&self, mut f: F) -> Result<Self, CircuitConstructionError>
+    fn map_children_enumerate<F>(&self, mut f: F) -> Result<Self>
     where
-        CircuitConstructionError: From<E>,
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc, E>,
+        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
     {
         Self::try_new(
             f(0, self.node.clone())?,
@@ -1403,11 +1353,7 @@ impl CircuitNode for Scatter {
         )
     }
 
-    fn eval_tensors(
-        &self,
-        tensors: &[Tensor],
-        _device_dtype: &TorchDeviceDtype,
-    ) -> Result<Tensor, super::TensorEvalError> {
+    fn eval_tensors(&self, tensors: &[Tensor], _device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
         Python::with_gil(|py| {
             PY_CIRCUIT_ITEMS
                 .circ_compiler_util
