@@ -1,14 +1,14 @@
-use rustc_hash::FxHashMap as HashMap;
 use std::iter::zip;
 
-use crate::pyo3_prelude::*;
 use num_bigint::BigUint;
+use rustc_hash::FxHashMap as HashMap;
 
 use super::{
     algebraic_rewrite::add_make_broadcasts_explicit, prelude::*, Add, Concat, Einsum,
     GeneralFunction, Index,
 };
 use crate::{
+    pyo3_prelude::*,
     tensor_util::{TensorAxisIndex, TensorIndex, USlice},
     util::{cumsum, outer_product, AsOp},
 };
