@@ -1,15 +1,4 @@
-use super::{
-    named_axes::set_named_axes, prelude::*, CachedCircuitInfo, PyCircuitBase, Shape,
-    TensorEvalError,
-};
-use crate::lru_cache::TensorCacheRrfs;
-use crate::rrfs::{arrayconstant_from_hash, arrayconstant_from_hash_prefix, save_tensor_rrfs};
-use crate::{
-    circuit_node_auto_impl, circuit_node_extra_impl,
-    py_types::{scalar_to_tensor, Tensor, PY_UTILS},
-    tensor_util::{TorchDeviceDtype, TorchDeviceDtypeOp},
-};
-use crate::{new_rc, pyo3_prelude::*, sv};
+use anyhow::Result;
 use base16::encode_lower;
 use macro_rules_attribute::apply;
 use num_bigint::BigUint;
@@ -17,7 +6,20 @@ use pyo3::types::{IntoPyDict, PyTuple};
 use rustc_hash::FxHashMap as HashMap;
 use uuid::Uuid;
 
-use crate::circuit::NamedAxes;
+use super::{
+    named_axes::set_named_axes, prelude::*, CachedCircuitInfo, NamedAxes, PyCircuitBase, Shape,
+    TensorEvalError,
+};
+use crate::{
+    circuit_node_auto_impl, circuit_node_extra_impl,
+    lru_cache::TensorCacheRrfs,
+    new_rc,
+    py_types::{scalar_to_tensor, Tensor, PY_UTILS},
+    pyo3_prelude::*,
+    rrfs::{arrayconstant_from_hash, arrayconstant_from_hash_prefix, save_tensor_rrfs},
+    sv,
+    tensor_util::{TorchDeviceDtype, TorchDeviceDtypeOp},
+};
 
 macro_rules! circuit_node_auto_leaf_impl {
     ($uuid:literal) => {
@@ -31,10 +33,9 @@ macro_rules! circuit_node_auto_leaf_impl {
             vec![]
         }
 
-        fn map_children_enumerate<F, E>(&self, _f: F) -> Result<Self, CircuitConstructionError>
+        fn map_children_enumerate<F>(&self, _f: F) -> Result<Self>
         where
-            CircuitConstructionError: From<E>,
-            F: FnMut(usize, CircuitRc) -> Result<CircuitRc, E>,
+            F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
         {
             Ok(self.clone())
         }
@@ -74,7 +75,7 @@ impl CircuitNode for ArrayConstant {
         &self,
         _tensors: &[Tensor],
         _device_dtype: &TorchDeviceDtype,
-    ) -> Result<Tensor, TensorEvalError> {
+    ) -> Result<Tensor> {
         Ok(self.value.clone())
     }
 
@@ -108,7 +109,7 @@ impl ArrayConstant {
         name: Option<String>,
         hash_base16: &str,
         tensor_cache: &mut Option<TensorCacheRrfs>,
-    ) -> Result<Self, PyErr> {
+    ) -> Result<Self> {
         arrayconstant_from_hash_prefix(name, hash_base16, tensor_cache)
     }
 }
@@ -184,7 +185,7 @@ impl ArrayConstant {
         Self::randn_named(shape, name, device_dtype)
     }
 
-    pub fn save_rrfs(&self) -> Result<String, PyErr> {
+    pub fn save_rrfs(&self) -> Result<String> {
         save_tensor_rrfs(self.value.clone())
     }
 
@@ -193,7 +194,7 @@ impl ArrayConstant {
     }
 
     #[staticmethod]
-    pub fn from_hash(name: Option<String>, hash_base16: &str) -> Result<Self, PyErr> {
+    pub fn from_hash(name: Option<String>, hash_base16: &str) -> Result<Self> {
         arrayconstant_from_hash(name, hash_base16)
     }
 
@@ -203,14 +204,14 @@ impl ArrayConstant {
         name: Option<String>,
         hash_base16: &str,
         tensor_cache: Option<TensorCacheRrfs>,
-    ) -> Result<Self, PyErr> {
+    ) -> Result<Self> {
         let mut tensor_cache = tensor_cache;
         arrayconstant_from_hash_prefix(name, hash_base16, &mut tensor_cache)
     }
 }
 
 #[pyclass(extends=PyCircuitBase, unsendable)]
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, PyClassDeriv)]
 pub struct Symbol {
     #[pyo3(get)]
     pub uuid: Uuid,
@@ -242,10 +243,11 @@ impl CircuitNode for Symbol {
         &self,
         _tensors: &[Tensor],
         _device_dtype: &TorchDeviceDtype,
-    ) -> Result<Tensor, TensorEvalError> {
+    ) -> Result<Tensor> {
         Err(TensorEvalError::NotExplicitlyComputable {
             circuit: self.crc(),
-        })
+        }
+        .into())
     }
 
     fn intermediate_cost_bound(&self) -> usize {
@@ -311,11 +313,7 @@ impl CircuitNode for ScalarConstant {
         hasher
     }
 
-    fn eval_tensors(
-        &self,
-        _tensors: &[Tensor],
-        device_dtype: &TorchDeviceDtype,
-    ) -> Result<Tensor, TensorEvalError> {
+    fn eval_tensors(&self, _tensors: &[Tensor], device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
         Ok(scalar_to_tensor(
             self.value,
             self.info().shape.clone(),
