@@ -1,8 +1,6 @@
-use rustc_hash::FxHashMap as HashMap;
-
-use crate::circuit::circuit_optimizer::OptimizationContext;
-use crate::circuit::HashBytes;
 use num_bigint::BigUint;
+use pyo3::prelude::*;
+use rustc_hash::FxHashMap as HashMap;
 
 use super::{
     circuit_manipulation::replace_nodes,
@@ -10,8 +8,10 @@ use super::{
     circuit_utils::toposort_circuit,
     evaluate_fn, ArrayConstant, Circuit, CircuitNode, CircuitRc,
 };
-use crate::py_types::{Tensor, PY_UTILS};
-use pyo3::prelude::*;
+use crate::{
+    circuit::{circuit_optimizer::OptimizationContext, HashBytes},
+    py_types::{Tensor, PY_UTILS},
+};
 
 /// We need this to redirect Python stdout to Rust stdout
 /// If we don't do this, then Python will print to its own stdout, which is not captured by Rust
