@@ -1,35 +1,34 @@
-use crate::circuit::batching::batch_einsum;
-use crate::circuit::generalfunction_rewrite::generalfunction_evaluate_simple;
-use crate::circuit::module_rewrite::elim_empty_module;
-use crate::circuit::scatter_rewrite::concat_to_scatter;
-use crate::circuit::{
-    deep_map_op_context, generalfunction_merge_inverses,
-    generalfunction_special_case_simplification,
-};
-use crate::util::{apply_fn_until_none, apply_fn_until_same, mapping_until_end, AsOp};
-use crate::{pyo3_prelude::*, timed, timed_value};
 use num_bigint::BigUint;
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
 
-use super::nb_rewrites::{add_elim_removable_axes_weak, einsum_elim_removable_axes_weak};
-use super::visit_circuit_unwrap;
 use super::{
     algebraic_rewrite::*,
     canonicalize::deep_canonicalize,
     circuit_optimizer::OptimizationContext,
-    concat_rewrite::add_pull_concat,
     concat_rewrite::{
-        concat_drop_size_zero, concat_fuse, einsum_pull_concat, generalfunction_pull_concat,
-        index_concat_drop_unreached,
+        add_pull_concat, concat_drop_size_zero, concat_fuse, einsum_pull_concat,
+        generalfunction_pull_concat, index_concat_drop_unreached,
     },
     deep_map_preorder_unwrap, deep_map_unwrap,
     diag_rewrite::{add_pull_diags, einsum_push_down_trace},
+    nb_rewrites::{add_elim_removable_axes_weak, einsum_elim_removable_axes_weak},
     prelude::*,
     scatter_rewrite::{
         add_pull_scatter, einsum_pull_scatter, index_einsum_to_scatter, scatter_elim_identity,
         scatter_fuse, scatter_pull_removable_axes,
     },
-    Add, Concat, GeneralFunction, Index,
+    visit_circuit_unwrap, Add, Concat, GeneralFunction, Index,
+};
+use crate::{
+    circuit::{
+        batching::batch_einsum, deep_map_op_context, generalfunction_merge_inverses,
+        generalfunction_rewrite::generalfunction_evaluate_simple,
+        generalfunction_special_case_simplification, module_rewrite::elim_empty_module,
+        scatter_rewrite::concat_to_scatter,
+    },
+    pyo3_prelude::*,
+    timed, timed_value,
+    util::{apply_fn_until_none, apply_fn_until_same, mapping_until_end, AsOp},
 };
 
 /// seperate _py function because pyfunctions cant take reference arguments
