@@ -1,22 +1,23 @@
-use crate::lazy::GILLazyPy;
-use crate::pyo3_prelude::*;
-use rustc_hash::FxHashMap as HashMap;
 use std::iter::zip;
 
+use anyhow::{Context, Result};
+use macro_rules_attribute::apply;
+use pyo3::{
+    once_cell::GILLazyPy,
+    types::{PyDict, PyTuple},
+};
+use rustc_hash::FxHashMap as HashMap;
+use uuid::uuid;
+
 use super::{prelude::*, CachedCircuitInfo, HashBytes};
-use crate::py_types::PY_UTILS;
 use crate::{
     circuit::PyCircuitBase,
     circuit_node_auto_impl, circuit_node_extra_impl, new_rc_unwrap,
-    py_types::py_address,
+    py_types::{py_address, PY_UTILS},
+    pyo3_prelude::*,
     tensor_util::{Shape, TorchDeviceDtype},
 };
 
-use macro_rules_attribute::apply;
-use pyo3::types::PyDict;
-use pyo3::types::PyTuple;
-use uuid::uuid;
-
 /// This only covers some cases - we should plausibly make a general version at
 /// some point.
 #[pyo3::prelude::pyclass] // this needs to be sent to python all the time
@@ -199,7 +200,7 @@ impl GeneralFunction {
         nodes: Vec<CircuitRc>,
         spec: GeneralFunctionSpec,
         name: Option<String>,
-    ) -> (Result<Self, CircuitConstructionError>) {
+    ) -> Result<Self> {
         let mut out = Self {
             nodes,
             spec,
@@ -208,10 +209,11 @@ impl GeneralFunction {
         };
         out.name = out.auto_name(name);
         if out.try_compute_shape().is_none() {
-            return Err(CircuitConstructionError::GeneralFunctionWrongInputShape {
+            return Err(ConstructError::GeneralFunctionWrongInputShape {
                 gf_spec: out.spec,
                 input_shapes: out.nodes.iter().map(|x| x.info().shape.clone()).collect(),
-            });
+            }
+            .into());
         }
         // todo check not too many axes / oob
         out.init_info()
@@ -265,10 +267,9 @@ impl CircuitNode for GeneralFunction {
         Box::new(self.nodes.iter().cloned())
     }
 
-    fn map_children_enumerate<F, E>(&self, mut f: F) -> Result<Self, CircuitConstructionError>
+    fn map_children_enumerate<F>(&self, mut f: F) -> Result<Self>
     where
-        CircuitConstructionError: From<E>,
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc, E>,
+        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
     {
         Self::try_new(
             self.nodes
@@ -303,17 +304,21 @@ impl CircuitNode for GeneralFunction {
         &self,
         tensors: &[crate::py_types::Tensor],
         _device_dtype: &TorchDeviceDtype,
-    ) -> Result<crate::py_types::Tensor, super::TensorEvalError> {
+    ) -> Result<crate::py_types::Tensor> {
         Python::with_gil(|py| {
-            self.spec
+            Ok(self
+                .spec
                 .function
                 .call(
                     py,
                     PyTuple::new(py, tensors.iter().map(|x| x.clone().into_py(py))),
                     None,
                 )
-                .map_err(|err| err.into())
-                .and_then(|r| r.extract(py).map_err(|e| e.into()))
+                .context(format!(
+                    "evaluate function {} (is_official={})",
+                    self.spec.name, self.spec.is_official
+                ))?
+                .extract(py)?)
         })
     }
 }
@@ -365,9 +370,9 @@ impl GeneralFunction {
         nodes: Vec<CircuitRc>,
         spec_name: String,
         name: Option<String>,
-    ) -> Result<Self, CircuitConstructionError> {
+    ) -> Result<Self> {
         Self::new_by_name_op(nodes, spec_name.clone(), name)?
-            .ok_or(CircuitConstructionError::UnknownGeneralFunction { spec_name })
+            .ok_or(ConstructError::UnknownGeneralFunction { spec_name }.into())
     }
 
     #[staticmethod]
@@ -375,7 +380,7 @@ impl GeneralFunction {
         nodes: Vec<CircuitRc>,
         spec_name: String,
         name: Option<String>,
-    ) -> Result<Option<Self>, CircuitConstructionError> {
+    ) -> Result<Option<Self>> {
         let spec = GENERAL_FUNCTIONS_SPECS.get(&spec_name);
         if spec.is_none() {
             return Ok(None);
