@@ -6,8 +6,9 @@ use std::{
     vec,
 };
 
+use anyhow::{bail, Result};
 use macro_rules_attribute::apply;
-use pyo3::{exceptions::PyTypeError, types::PyTuple, AsPyPointer};
+use pyo3::{types::PyTuple, AsPyPointer};
 use uuid::uuid;
 
 use super::{
@@ -15,7 +16,7 @@ use super::{
     Transform, TransformRc, Updater,
 };
 use crate::{
-    circuit::{CircuitConstructionError, CircuitRc, HashBytes},
+    circuit::{CircuitRc, HashBytes},
     eq_by_big_hash::EqByBigHash,
     impl_both_by_big_hash,
     pyo3_prelude::*,
@@ -235,7 +236,7 @@ impl FilterIterativeMatcher {
         }
     }
 
-    pub fn match_iterate(&self, circuit: CircuitRc) -> PyResult<IterateMatchResults> {
+    pub fn match_iterate(&self, circuit: CircuitRc) -> Result<IterateMatchResults> {
         let with_dist_of_end = |offset: u32| {
             self.end_depth
                 .map(|x| self.depth >= x.saturating_sub(offset))
@@ -354,7 +355,7 @@ impl IterativeMatcher {
         Self::any(vec![self.crc(), other])
     }
 
-    pub fn validate_matched(&self, matched: &HashSet<CircuitRc>) -> PyResult<()> {
+    pub fn validate_matched(&self, matched: &HashSet<CircuitRc>) -> Result<()> {
         match &self.data {
             IterativeMatcherData::Match(m) => m.validate_matched(matched),
             IterativeMatcherData::Filter(m) => m.iterative_matcher.validate_matched(matched),
@@ -382,7 +383,7 @@ impl IterativeMatcher {
         }
     }
 
-    pub fn match_iterate(&self, circuit: CircuitRc) -> PyResult<IterateMatchResults> {
+    pub fn match_iterate(&self, circuit: CircuitRc) -> Result<IterateMatchResults> {
         let res = match &self.data {
             IterativeMatcherData::Match(m) => IterateMatchResults {
                 updated: None,
@@ -421,7 +422,7 @@ impl IterativeMatcher {
                     chain: MaybeChain<'_>,
                     circuit: &CircuitRc,
                     new_items: &mut HashSet<ChainItem>,
-                ) -> PyResult<bool> {
+                ) -> Result<bool> {
                     let IterateMatchResults {
                         updated,
                         finished,
@@ -482,7 +483,7 @@ impl IterativeMatcher {
         Ok(res)
     }
 
-    fn validate_matched_py(&self, matched: HashSet<CircuitRc>) -> PyResult<()> {
+    fn validate_matched_py(&self, matched: HashSet<CircuitRc>) -> Result<()> {
         self.validate_matched(&matched)
     }
 
@@ -506,17 +507,15 @@ impl IterativeMatcher {
 
     #[staticmethod]
     #[args(chains = "*")]
-    pub fn mk_chain_many(chains: Vec<Vec<IterativeMatcherRc>>) -> PyResult<Self> {
+    pub fn mk_chain_many(chains: Vec<Vec<IterativeMatcherRc>>) -> Result<Self> {
         Ok(IterativeMatcherData::Chains(
             chains
                 .into_iter()
                 .map(|mut chain| match split_first_take(&mut chain) {
-                    None => Err(PyTypeError::new_err(
-                        "Received empty tuple for a chain, we expect len >= 1",
-                    )),
+                    None => bail!("Received empty tuple for a chain, we expect len >= 1",),
                     Some((first, rest)) => Ok(ChainItem::new(first, rest.collect())),
                 })
-                .collect::<PyResult<_>>()?,
+                .collect::<Result<_>>()?,
         )
         .into())
     }
@@ -625,7 +624,7 @@ impl IterativeMatcher {
         term_if_matches: bool,
         depth_slice: Slice,
         term_early_at: MatcherRc,
-    ) -> PyResult<IterativeMatcher> {
+    ) -> Result<IterativeMatcher> {
         Ok(self_.filter(
             term_if_matches,
             flip_op_result(depth_slice.start.map(|x| x.try_into()))?,
@@ -657,7 +656,7 @@ impl IterativeMatcher {
     }
 
     #[args(fancy_validate = "Getter::default().default_fancy_validate")]
-    pub fn get(&self, circuit: CircuitRc, fancy_validate: bool) -> PyResult<HashSet<CircuitRc>> {
+    pub fn get(&self, circuit: CircuitRc, fancy_validate: bool) -> Result<HashSet<CircuitRc>> {
         Getter::default().get(circuit, self_.crc(), Some(fancy_validate))
     }
 
@@ -666,16 +665,16 @@ impl IterativeMatcher {
         &self,
         circuit: CircuitRc,
         fancy_validate: bool,
-    ) -> PyResult<Option<CircuitRc>> {
+    ) -> Result<Option<CircuitRc>> {
         Getter::default().get_unique_op(circuit, self_.crc(), Some(fancy_validate))
     }
 
     #[args(fancy_validate = "Getter::default().default_fancy_validate")]
-    pub fn get_unique(&self, circuit: CircuitRc, fancy_validate: bool) -> PyResult<CircuitRc> {
+    pub fn get_unique(&self, circuit: CircuitRc, fancy_validate: bool) -> Result<CircuitRc> {
         Getter::default().get_unique(circuit, self_.crc(), Some(fancy_validate))
     }
 
-    pub fn validate(&self, circuit: CircuitRc) -> PyResult<()> {
+    pub fn validate(&self, circuit: CircuitRc) -> Result<()> {
         Getter::default().validate(circuit, self_.crc())
     }
 
@@ -700,7 +699,7 @@ impl IterativeMatcher {
         cache_transform_along_modified_path: bool,
         cache_update: bool,
         fancy_validate: bool,
-    ) -> Result<CircuitRc, CircuitConstructionError> {
+    ) -> Result<CircuitRc> {
         Updater::new(
             transform,
             transform_along_modified_path,
