@@ -3,6 +3,7 @@ use std::{
     sync::Arc,
 };
 
+use anyhow::Result;
 use pyo3::AsPyPointer;
 use uuid::uuid;
 
@@ -67,7 +68,7 @@ impl Transform {
         inp.into()
     }
 
-    pub fn run(&self, circuit: CircuitRc) -> PyResult<CircuitRc> {
+    pub fn run(&self, circuit: CircuitRc) -> Result<CircuitRc> {
         let ret = match &self.data {
             TransformData::Raw(f) => f.0(circuit)?,
             TransformData::Ident => circuit.clone(),
