@@ -1,20 +1,14 @@
 #![allow(clippy::borrow_deref_ref)]
-use crate::all_imports::{RInnerInts, Sv, TensorInvariantError};
-use crate::circuit::circuit_utils::toposort_circuit;
-use crate::py_types::use_rust_comp;
-use crate::pyo3_prelude::*;
-use crate::util::{arc_ref_clone, arc_unwrap_or_clone, AxisInt};
-use crate::{
-    cached_lambda,
-    eq_by_big_hash::EqByBigHash,
-    py_types::{PyShape, Tensor},
-    rearrange_spec::{OpSize, RearrangeSpec},
-    tensor_util::{
-        Shape, Slice, TensorAxisIndex, TensorIndex, TorchDeviceDtype, TorchDeviceDtypeOp,
-    },
-    util::AsOp,
+use std::{
+    collections::BTreeMap,
+    fmt::Debug,
+    hash::Hash,
+    iter::zip,
+    ops::{Deref, DerefMut},
+    sync::Arc,
 };
-use crate::{pycall, pycallable, sv};
+
+use anyhow::Result;
 pub use computational_node::{
     flat_concat, flat_concat_back, Add, Concat, Einsum, EinsumAxes, Index, Rearrange, Scatter,
 };
@@ -24,27 +18,36 @@ pub use generalfunction::{GeneralFunction, GeneralFunctionSpec, PyGFSpecShapeGet
 pub use generalfunction_rewrite::{
     generalfunction_merge_inverses, generalfunction_special_case_simplification,
 };
+use macro_rules_attribute::apply;
 pub use module_nodes::{ModuleNode, ModuleNodeArgSpec, ModuleNodeSpec};
 use num_bigint::BigUint;
-pub use parsing::{parse_compiler_repr_bijection, parse_compiler_repr_bijection_py};
+pub use parsing::{parse_compiler_repr_bijection, parse_compiler_repr_bijection_py, ParseError};
 use py_circuit_items::circuit_rust_to_py;
-use pyo3::types::PyBytes;
-use pyo3::PyTypeInfo;
-use pyo3::{exceptions, pyclass::CompareOp};
-use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
-use std::collections::BTreeMap;
-use std::hash::Hash;
-use std::sync::Arc;
-use std::{
-    iter::zip,
-    ops::{Deref, DerefMut},
+use pyo3::{
+    exceptions::{self, PyValueError},
+    pyclass::CompareOp,
+    types::PyBytes,
+    PyTypeInfo,
 };
+use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
+use thiserror::Error;
 use uuid::uuid;
 pub use variable_nodes::{AutoTag, DiscreteVar, StoredCumulantVar};
 
-use macro_rules_attribute::apply;
-use std::fmt::Debug;
-use thiserror::Error;
+use crate::{
+    all_imports::{RInnerInts, Sv},
+    cached_lambda,
+    circuit::circuit_utils::toposort_circuit,
+    eq_by_big_hash::EqByBigHash,
+    py_types::{use_rust_comp, PyShape, Tensor},
+    pycall, pycallable,
+    pyo3_prelude::*,
+    python_error_exception,
+    rearrange_spec::{OpSize, RearrangeSpec},
+    sv,
+    tensor_util::{Shape, TensorAxisIndex, TensorIndex, TorchDeviceDtype, TorchDeviceDtypeOp},
+    util::{arc_ref_clone, arc_unwrap_or_clone, AsOp, AxisInt},
+};
 
 pub mod algebraic_rewrite;
 pub mod batching;
@@ -61,6 +64,7 @@ pub mod cumulant;
 pub mod debugging;
 pub mod deep_rewrite;
 pub mod diag_rewrite;
+pub mod expand_node;
 pub mod generalfunction;
 pub mod generalfunction_rewrite;
 pub mod get_update;
@@ -81,14 +85,14 @@ pub mod scheduling_z3;
 pub mod variable_nodes;
 
 mod circuit_node_private {
-    use super::{CachedCircuitInfo, CircuitConstructionError};
+    use super::{CachedCircuitInfo, Result};
     pub trait CircuitNodePrivate {
         fn info_mut(&mut self) -> &mut CachedCircuitInfo;
         fn name_mut(&mut self) -> &mut Option<String>;
     }
 
     pub trait CircuitNodeInit {
-        fn init_info_impl(self) -> Result<Self, CircuitConstructionError>
+        fn init_info_impl(self) -> Result<Self>
         where
             Self: Sized;
 
@@ -96,7 +100,7 @@ mod circuit_node_private {
         where
             Self: Sized;
 
-        fn update_info_impl<F>(self, f: F) -> Result<Self, CircuitConstructionError>
+        fn update_info_impl<F>(self, f: F) -> Result<Self>
         where
             Self: Sized,
             F: FnOnce(&mut CachedCircuitInfo);
@@ -107,7 +111,7 @@ use circuit_node_private::*;
 use self::circuit_utils::total_arrayconstant_size;
 
 impl<T: CircuitNodePrivate + CircuitNode> CircuitNodeInit for T {
-    fn init_info_impl(mut self) -> Result<Self, CircuitConstructionError>
+    fn init_info_impl(mut self) -> Result<Self>
     where
         Self: Sized,
     {
@@ -120,7 +124,7 @@ impl<T: CircuitNodePrivate + CircuitNode> CircuitNodeInit for T {
         hasher.update(uuid!("025e9af4-1366-4211-aa5f-7c28fc6cdf9f").as_bytes());
         for (axis, name) in &self.info().named_axes {
             if *axis as usize >= self.info().shape.len() {
-                return Err(CircuitConstructionError::NamedAxisAboveRank {});
+                return Err(ConstructError::NamedAxisAboveRank {}.into());
             }
             hasher.update(&[*axis]);
             hasher.update(name.as_bytes());
@@ -142,7 +146,7 @@ impl<T: CircuitNodePrivate + CircuitNode> CircuitNodeInit for T {
         self.init_info_impl().unwrap() // we could avoid recomputing some stuff if we wanted
     }
 
-    fn update_info_impl<F>(mut self, f: F) -> Result<Self, CircuitConstructionError>
+    fn update_info_impl<F>(mut self, f: F) -> Result<Self>
     where
         Self: Sized,
         F: FnOnce(&mut CachedCircuitInfo),
@@ -205,11 +209,10 @@ pub trait CircuitNode: CircuitNodeInit {
 
     fn children<'a>(&'a self) -> Box<dyn Iterator<Item = CircuitRc> + 'a>;
 
-    fn map_children_enumerate<F, E>(&self, f: F) -> Result<Self, CircuitConstructionError>
+    fn map_children_enumerate<F>(&self, f: F) -> Result<Self>
     where
         Self: Sized,
-        CircuitConstructionError: From<E>,
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc, E>;
+        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>;
 
     fn node_type_uuid(&self) -> [u8; 16];
 
@@ -217,11 +220,7 @@ pub trait CircuitNode: CircuitNodeInit {
         BigUint::from(0usize)
     }
 
-    fn eval_tensors(
-        &self,
-        tensors: &[Tensor],
-        device_dtype: &TorchDeviceDtype,
-    ) -> Result<Tensor, TensorEvalError>;
+    fn eval_tensors(&self, tensors: &[Tensor], device_dtype: &TorchDeviceDtype) -> Result<Tensor>;
 
     /// At most how many elements will evaluating this circuit require allocating
     /// new memory (that we are allowed to free ourselves) for? Used to improve scheduling.
@@ -231,7 +230,7 @@ pub trait CircuitNode: CircuitNodeInit {
 
     fn c(self) -> Circuit;
     fn rc(self) -> CircuitRc;
-    fn init_info(self) -> Result<Self, CircuitConstructionError>
+    fn init_info(self) -> Result<Self>
     where
         Self: Sized,
     {
@@ -243,7 +242,7 @@ pub trait CircuitNode: CircuitNodeInit {
     {
         self.rename_impl(new_name)
     }
-    fn update_info<F>(self, f: F) -> Result<Self, CircuitConstructionError>
+    fn update_info<F>(self, f: F) -> Result<Self>
     where
         Self: Sized,
         F: FnOnce(&mut CachedCircuitInfo),
@@ -265,7 +264,7 @@ pub trait CircuitNode: CircuitNodeInit {
         self.name().map(|x| x.to_owned())
     }
 
-    fn compute_device_dtype(&self) -> Result<TorchDeviceDtypeOp, CircuitConstructionError> {
+    fn compute_device_dtype(&self) -> Result<TorchDeviceDtypeOp> {
         self.children()
             .map(|c| c.info().device_dtype.clone())
             .chain(self.device_dtype_extra())
@@ -293,20 +292,18 @@ pub trait CircuitNode: CircuitNodeInit {
         result
     }
 
-    fn map_children<F, E>(&self, mut f: F) -> Result<Self, CircuitConstructionError>
+    fn map_children<F>(&self, mut f: F) -> Result<Self>
     where
         Self: Sized,
-        CircuitConstructionError: From<E>,
-        F: FnMut(CircuitRc) -> Result<CircuitRc, E>,
+        F: FnMut(CircuitRc) -> Result<CircuitRc>,
     {
         self.map_children_enumerate(|_i, x| f(x))
     }
 
-    fn map_children_idxs<F, E>(&self, mut f: F) -> Result<Self, CircuitConstructionError>
+    fn map_children_idxs<F>(&self, mut f: F) -> Result<Self>
     where
         Self: Sized,
-        CircuitConstructionError: From<E>,
-        F: FnMut(usize) -> Result<CircuitRc, E>,
+        F: FnMut(usize) -> Result<CircuitRc>,
     {
         self.map_children_enumerate(|i, _x| f(i))
     }
@@ -316,8 +313,7 @@ pub trait CircuitNode: CircuitNodeInit {
         Self: Sized,
         F: FnMut(CircuitRc) -> CircuitRc,
     {
-        self.map_children(|x| Ok::<CircuitRc, CircuitConstructionError>(f(x)))
-            .unwrap()
+        self.map_children(|x| Ok(f(x))).unwrap()
     }
 
     fn map_children_unwrap_enumerate<F>(&self, mut f: F) -> Self
@@ -325,8 +321,7 @@ pub trait CircuitNode: CircuitNodeInit {
         Self: Sized,
         F: FnMut(usize, CircuitRc) -> CircuitRc,
     {
-        self.map_children_enumerate(|i, x| Ok::<CircuitRc, CircuitConstructionError>(f(i, x)))
-            .unwrap()
+        self.map_children_enumerate(|i, x| Ok(f(i, x))).unwrap()
     }
 
     fn map_children_unwrap_idxs<F>(&self, mut f: F) -> Self
@@ -334,8 +329,7 @@ pub trait CircuitNode: CircuitNodeInit {
         Self: Sized,
         F: FnMut(usize) -> CircuitRc,
     {
-        self.map_children_enumerate(|i, _x| Ok::<CircuitRc, CircuitConstructionError>(f(i)))
-            .unwrap()
+        self.map_children_enumerate(|i, _x| Ok(f(i))).unwrap()
     }
 
     /// if any return Some, return child mapped, otherwise None
@@ -386,16 +380,19 @@ pub trait CircuitNode: CircuitNodeInit {
         self.info().hash
     }
 
-    fn check_canon_axes(&self, axes: &[i64]) -> Result<RInnerInts, CircuitConstructionError> {
+    fn check_canon_axes(&self, axes: &[i64]) -> Result<RInnerInts> {
         check_canon_idxs(self.info().rank(), axes)
             .map(|v| v.into_iter().map(|x| x as u8).collect())
-            .map_err(|axis| CircuitConstructionError::ReductionAxisOutOfBounds {
-                axis,
-                node_rank: self.info().rank(),
+            .map_err(|axis| {
+                ConstructError::ReductionAxisOutOfBounds {
+                    axis,
+                    node_rank: self.info().rank(),
+                }
+                .into()
             })
     }
 
-    fn sum(&self, axes: &[i64], name: Option<String>) -> Result<Einsum, CircuitConstructionError>
+    fn sum(&self, axes: &[i64], name: Option<String>) -> Result<Einsum>
     where
         Self: Clone,
     {
@@ -409,7 +406,7 @@ pub trait CircuitNode: CircuitNodeInit {
         ))
     }
 
-    fn mean(&self, axes: &[i64], name: Option<String>) -> Result<Einsum, CircuitConstructionError>
+    fn mean(&self, axes: &[i64], name: Option<String>) -> Result<Einsum>
     where
         Self: Clone,
     {
@@ -424,12 +421,7 @@ pub trait CircuitNode: CircuitNodeInit {
         )
     }
 
-    fn reduce(
-        &self,
-        op_name: String,
-        axes: &[i64],
-        name: Option<String>,
-    ) -> Result<Circuit, CircuitConstructionError>
+    fn reduce(&self, op_name: String, axes: &[i64], name: Option<String>) -> Result<Circuit>
     where
         Self: Clone,
     {
@@ -453,35 +445,31 @@ pub trait CircuitNode: CircuitNodeInit {
         .unwrap()
         .c())
     }
-    fn min_(&self, axes: &[i64], name: Option<String>) -> Result<Circuit, CircuitConstructionError>
+    fn min_(&self, axes: &[i64], name: Option<String>) -> Result<Circuit>
     where
         Self: Clone,
     {
         self.reduce("min".to_owned(), axes, name)
     }
-    fn max_(&self, axes: &[i64], name: Option<String>) -> Result<Circuit, CircuitConstructionError>
+    fn max_(&self, axes: &[i64], name: Option<String>) -> Result<Circuit>
     where
         Self: Clone,
     {
         self.reduce("max".to_owned(), axes, name)
     }
-    fn add(&self, other: CircuitRc, name: Option<String>) -> Result<Add, CircuitConstructionError>
+    fn add(&self, other: CircuitRc, name: Option<String>) -> Result<Add>
     where
         Self: Clone,
     {
         Add::try_new(vec![self.crc(), other], name)
     }
-    fn sub(&self, other: CircuitRc, name: Option<String>) -> Result<Add, CircuitConstructionError>
+    fn sub(&self, other: CircuitRc, name: Option<String>) -> Result<Add>
     where
         Self: Clone,
     {
         self.add(Einsum::scalar_mul(other, -1.0, None).rc(), name)
     }
-    fn mul(
-        &self,
-        other: CircuitRc,
-        name: Option<String>,
-    ) -> Result<Einsum, CircuitConstructionError>
+    fn mul(&self, other: CircuitRc, name: Option<String>) -> Result<Einsum>
     where
         Self: Clone,
     {
@@ -492,17 +480,13 @@ pub trait CircuitNode: CircuitNodeInit {
         scalar: f64,
         name: Option<String>,
         scalar_name: Option<String>,
-    ) -> Result<Einsum, CircuitConstructionError>
+    ) -> Result<Einsum>
     where
         Self: Clone,
     {
         self.mul(ScalarConstant::nrc(scalar, sv![], scalar_name), name)
     }
-    fn index(
-        &self,
-        index: TensorIndex,
-        name: Option<String>,
-    ) -> Result<Index, CircuitConstructionError>
+    fn index(&self, index: TensorIndex, name: Option<String>) -> Result<Index>
     where
         Self: Clone,
     {
@@ -534,11 +518,10 @@ impl<T: CircuitNode> EqByBigHash for T {
 
 pub trait CircuitNodeDefer: CircuitNodeInit {
     fn as_trait_obj(&self) -> &dyn CircuitNode;
-    fn map_children_enumerate_impl<F, E>(&self, f: F) -> Result<Self, CircuitConstructionError>
+    fn map_children_enumerate_impl<F>(&self, f: F) -> Result<Self>
     where
         Self: Sized,
-        CircuitConstructionError: From<E>,
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc, E>;
+        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>;
     fn custom_c(self) -> Circuit;
     fn custom_rc(self) -> CircuitRc;
 }
@@ -690,7 +673,7 @@ macro_rules! define_circuit_union_impl {
         }
 
         impl $crate::circuit::CircuitNodeInit for $name {
-            fn init_info_impl(self) -> Result<Self, $crate::circuit::CircuitConstructionError> {
+            fn init_info_impl(self) -> Result<Self> {
                 match self {
                     $(
                         Self::$x(node) => Ok(Self::$x(node.init_info()?)),
@@ -706,7 +689,7 @@ macro_rules! define_circuit_union_impl {
                 }
             }
 
-            fn update_info_impl<F>(self, f: F) -> Result<Self, $crate::circuit::CircuitConstructionError>
+            fn update_info_impl<F>(self, f: F) -> Result<Self>
             where
                 F: FnOnce(&mut $crate::circuit::CachedCircuitInfo),
             {
@@ -728,10 +711,9 @@ macro_rules! define_circuit_union_impl {
                 }
             }
 
-            fn map_children_enumerate_impl< F, E>(&self, f: F) -> Result<Self, $crate::circuit::CircuitConstructionError>
+            fn map_children_enumerate_impl<F>(&self, f: F) -> Result<Self>
             where
-                $crate::circuit::CircuitConstructionError: From<E>,
-                F: FnMut(usize,$crate::circuit::CircuitRc) -> Result<$crate::circuit::CircuitRc, E>,
+                F: FnMut(usize,$crate::circuit::CircuitRc) -> Result<$crate::circuit::CircuitRc>,
             {
                 match self {
                     $(
@@ -1019,10 +1001,9 @@ impl<T: CircuitNodeDefer> CircuitNode for T {
         self.as_trait_obj().children()
     }
 
-    fn map_children_enumerate<F, E>(&self, f: F) -> Result<Self, CircuitConstructionError>
+    fn map_children_enumerate<F>(&self, f: F) -> Result<Self>
     where
-        CircuitConstructionError: From<E>,
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc, E>,
+        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
     {
         self.map_children_enumerate_impl(f)
     }
@@ -1035,11 +1016,7 @@ impl<T: CircuitNodeDefer> CircuitNode for T {
         self.as_trait_obj().self_flops()
     }
 
-    fn eval_tensors(
-        &self,
-        tensors: &[Tensor],
-        device_dtype: &TorchDeviceDtype,
-    ) -> Result<Tensor, TensorEvalError> {
+    fn eval_tensors(&self, tensors: &[Tensor], device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
         self.as_trait_obj().eval_tensors(tensors, device_dtype)
     }
 
@@ -1100,8 +1077,11 @@ impl CachedCircuitInfo {
     }
 }
 
+#[apply(python_error_exception)]
+#[base_error_name(Construct)]
+#[base_exception(PyValueError)]
 #[derive(Error, Debug, Clone)]
-pub enum CircuitConstructionError {
+pub enum ConstructError {
     #[error("DiscreteVar doesn't have leading 'samples' dim")]
     DiscreteVarNoSamplesDim {},
 
@@ -1116,8 +1096,6 @@ pub enum CircuitConstructionError {
 
     #[error("StoredCumulantVar invalid cumulant number {number}")]
     StoredCumulantVarInvalidCumulantNumber { number: usize },
-    #[error("TensorInvariantError {err:?}")]
-    TensorInvariantError { err: TensorInvariantError },
 
     #[error("StoredCumulantVar cumulant number {cumulant_number} needs to be base shape, {base_shape:?} times cumulant number, got {cumulant_shape:?}")]
     StoredCumulantVarCumulantWrongShape {
@@ -1146,9 +1124,6 @@ pub enum CircuitConstructionError {
     #[error("Einsum string invalid {string} {substring}")]
     EinsumStringInvalid { string: String, substring: String },
 
-    #[error("Sum nodes not broadcastable, {shapes:?}")]
-    SumNotBroadcastable { shapes: Vec<Shape> },
-
     #[error("Rearrange takes different input shape, shape: {shape:?} spec: {spec:?}")]
     RearrangeWrongInputShape { spec: RearrangeSpec, shape: Shape },
 
@@ -1185,9 +1160,6 @@ pub enum CircuitConstructionError {
         l: usize,
     },
 
-    #[error("Start comes after its stop in slice {s:?}.")]
-    SliceDisordered { s: Slice },
-
     #[error("Scatter shape wrong, index: {index_shape:?} child: {shape:?} {index:?}")]
     ScatterShapeWrong {
         index: TensorIndex,
@@ -1195,11 +1167,8 @@ pub enum CircuitConstructionError {
         index_shape: Shape,
     },
 
-    #[error("Scatter not supported yet, {index:?}")]
-    ScatterUnimplemented { index: TensorIndex },
-
-    #[error("num_batches {num_batches} doesn't divide length {l}")]
-    BatchNumberDoesntDivide { l: usize, num_batches: usize },
+    #[error("This scatter index not supported yet, {index:?}")]
+    ScatterIndexTypeUnimplemented { index: TensorIndex },
 
     #[error("Children multiple dtypes {a:?} {b:?}")]
     ChildrenMultipleDtypes {
@@ -1219,172 +1188,30 @@ pub enum CircuitConstructionError {
     #[error("ModuleNode wrong number of children, expected {expected} got {got}")]
     ModuleNodeWrongNumberChildren { expected: usize, got: usize },
 
-    #[error("ModuleNode incompatible shapes, got {got:?} default {default:?}")]
-    ModuleNodeIncompatibleShapes {
-        got: Vec<Shape>,
-        default: Vec<Shape>,
-    },
-
-    #[error("ModuleNode expansion error {error}")]
-    ModuleNodeExpansionError {
-        error: Box<CircuitConstructionError>,
-    },
-
-    #[error("Batching Rank Too Low")]
-    BatchingRankTooLow {
-        default: Vec<usize>,
-        got: Vec<usize>,
-    },
-
-    #[error("Trying to expand fixed index, index {index:?} old shape{old_shape:?} new shape {new_shape:?}")]
-    ExpandingFixedIndex {
-        index: TensorIndex,
-        old_shape: Shape,
-        new_shape: Shape,
-    },
-
-    #[error(
-        "Trying to expand concat axis, index {axis} old shape{old_shape:?} new shape {new_shape:?}"
-    )]
-    ExpandingConcatAxis {
-        axis: usize,
-        old_shape: Shape,
-        new_shape: Shape,
-    },
-
-    #[error("Inputs that should have same batching have different batchings, {batch_shapes:?} {circuit:?}")]
-    InconsistentBatches {
-        batch_shapes: Vec<Shape>,
-        circuit: CircuitRc,
-    },
-
-    #[error("Would need to batch multiple axes, only supports one")]
-    BatchingRequiresMultipleAxes {},
-
     #[error("ModuleNode got unknown keyword argument, {argument}")]
     ModuleNodeUnknownArgument { argument: String },
 
-    #[error("Parsing no regex match '{line}'")]
-    ParsingNoRegexMatch { line: String },
-
-    #[error("Parsing number failed {string}")]
-    ParsingNumberFail { string: String },
-
-    #[error("Parsing failed {string}")]
-    ParsingFail { string: String },
-
-    #[error("Expected UUID string, found  {string}")]
-    InvalidUuid { string: String },
-
-    #[error("Parsing wrong number of children, expected {expected} found {found}")]
-    ParsingWrongNumberChildren { expected: usize, found: usize },
-
-    #[error("Parsing invalid indentation, tab width {tab_width} num spaces {spaces} stack indentation {stack_indentation} stack top {stack_top:?}")]
-    ParsingInvalidIndentation {
-        spaces: usize,
-        tab_width: usize,
-        stack_indentation: usize,
-        stack_top: Option<String>,
-    },
-
-    #[error("Parsing invalid circuit variant {v}")]
-    ParsingInvalidVariant { v: String },
-
-    #[error("Parsing invalid serial number {serial_number}")]
-    ParsingInvalidSerialNumber { serial_number: usize },
-
-    #[error("Tensor hash not found {hash}")]
-    TensorHashNotFound { hash: String },
-
-    #[error("Parsing Extra Unneeded String '{string}'")]
-    ParsingExtraUnneededString { string: String },
-
-    #[error("Parsing shape needed but not provided on {variant}")]
-    ParsingShapeNeeded { variant: String },
-
-    #[error("Parsing found multiple circuits in sequence, instead of one circuit (aka last node wasn't child of anything)")]
-    ParsingGotMultipleCircuits {},
-
     #[error("Named axis higher than rank")]
     NamedAxisAboveRank {},
 
-    #[error("trying to expand node, unknown variant {variant}")]
-    ExpandNodeUnknownVariant { variant: String },
-
-    #[error("Batching axis originates too high")]
-    BatchAxisOriginatesTooHigh {},
-
-    #[error("ExpandingRemovableAxisUnfortunateError")]
-    ExpandingRemovableAxisUnfortunateError {},
-
-    #[error("Bug Error (yikes!) (aka unreachable code / assert failure)")]
-    BugError {},
-
-    #[error("Module by name not found: {name}")]
-    ModuleNotFound { name: String },
-
-    #[error("Circuit by name not found: {name}")]
-    CircuitRefNotFound { name: String },
-
     #[error("Failed to construct equivalent explicitly computable circuit")]
     NoEquivalentExplicitlyComputable {},
-
-    #[error("python error {py_err}")]
-    PythonError { py_err: Arc<PyErr> },
-
-    #[error("just stopping iteration, not an actual error")]
-    StopIteration {},
 }
 
-impl From<CircuitConstructionError> for PyErr {
-    fn from(err: CircuitConstructionError) -> Self {
-        match err {
-            CircuitConstructionError::PythonError { py_err } => Python::with_gil(|py| {
-                let err = (*py_err).value(py);
-                err.into()
-            }),
-            _ => PyErr::new::<exceptions::PyValueError, _>(format!("error (TODO: better) {}", err)),
-        }
-    }
-}
-
-impl From<PyErr> for CircuitConstructionError {
-    fn from(py_err: PyErr) -> Self {
-        CircuitConstructionError::PythonError {
-            py_err: Arc::new(py_err),
-        }
-    }
-}
-
-pub type CircResult = Result<CircuitRc, CircuitConstructionError>;
+pub type CircResult = Result<CircuitRc>;
 
+#[apply(python_error_exception)]
+#[base_error_name(TensorEval)]
+#[base_exception(PyValueError)]
 #[derive(Error, Debug, Clone)]
 pub enum TensorEvalError {
     #[error("not explicitly computable: {circuit:?})")]
     NotExplicitlyComputable { circuit: CircuitRc },
-    #[error("python error {py_err:?}")]
-    PythonError { py_err: Arc<PyErr> }, // PyErr doesn't have .clone, so Rc-ing
     #[error("incompatible dtype circ:{circ:?} passed:{passed:?}")]
     DeviceDtypeError {
         circ: TorchDeviceDtypeOp,
         passed: TorchDeviceDtypeOp,
     },
-    #[error("Failed to construct equivalent explicitly computable circuit")]
-    NoEquivalentExplicitlyComputable {},
-}
-
-impl From<TensorEvalError> for PyErr {
-    fn from(err: TensorEvalError) -> Self {
-        PyErr::new::<exceptions::PyValueError, _>(format!("{}", err))
-    }
-}
-
-impl From<PyErr> for TensorEvalError {
-    fn from(py_err: PyErr) -> Self {
-        TensorEvalError::PythonError {
-            py_err: Arc::new(py_err),
-        }
-    }
 }
 
 #[derive(Clone, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
@@ -1396,14 +1223,14 @@ pub fn make_children_zero<T: CircuitNode>(circuit: &T) -> T {
     })
 }
 
-pub fn evaluate_fn(circ: CircuitRc) -> Result<Tensor, TensorEvalError> {
+pub fn evaluate_fn(circ: CircuitRc) -> Result<Tensor> {
     evaluate_fn_dtype_device(circ, Default::default())
 }
 
 pub fn evaluate_fn_dtype_device(
     circ: CircuitRc,
     dtype_device: TorchDeviceDtypeOp,
-) -> Result<Tensor, TensorEvalError> {
+) -> Result<Tensor> {
     let device_dtype = dtype_device
         .clone()
         .combine(circ.info().device_dtype.clone())
@@ -1414,9 +1241,9 @@ pub fn evaluate_fn_dtype_device(
         .unwrap_or_defaults();
     #[apply(cached_lambda)]
     #[key(circ.info().hash, HashBytes)]
-    fn recurse(circ: CircuitRc) -> Result<Tensor, TensorEvalError> {
-        let child_tensors: Result<Vec<Tensor>, TensorEvalError> =
-            circ.children().map(recurse).collect();
+    #[use_try]
+    fn recurse(circ: CircuitRc) -> Result<Tensor> {
+        let child_tensors: Result<Vec<Tensor>> = circ.children().map(recurse).collect();
         let child_tensors = child_tensors?;
 
         circ.eval_tensors(&child_tensors, &device_dtype)
@@ -1427,13 +1254,14 @@ pub fn evaluate_fn_dtype_device(
 
 #[apply(pycallable)]
 #[pyo3(name = "deep_map")]
-pub fn deep_map<F>(circuit: CircuitRc, f: F) -> Result<CircuitRc, CircuitConstructionError>
+pub fn deep_map<F>(circuit: CircuitRc, f: F) -> Result<CircuitRc>
 where
-    F: Fn((circuit, CircuitRc)) -> Result<CircuitRc, _>,
+    F: Fn((circuit, CircuitRc)) -> Result<CircuitRc>,
 {
     #[apply(cached_lambda)]
     #[key(circ.info().hash, HashBytes)]
-    fn recurse(circ: CircuitRc) -> Result<CircuitRc, CircuitConstructionError> {
+    #[use_try]
+    fn recurse(circ: CircuitRc) -> Result<CircuitRc> {
         let inner_mapped = circ.map_children(&mut recurse)?.rc();
         f(inner_mapped)
     }
@@ -1442,13 +1270,14 @@ where
 
 #[apply(pycallable)]
 #[pyo3(name = "deep_map_preorder")]
-pub fn deep_map_preorder<F>(circuit: CircuitRc, f: F) -> Result<CircuitRc, CircuitConstructionError>
+pub fn deep_map_preorder<F>(circuit: CircuitRc, f: F) -> Result<CircuitRc>
 where
-    F: Fn((circuit, CircuitRc)) -> Result<CircuitRc, _>,
+    F: Fn((circuit, CircuitRc)) -> Result<CircuitRc>,
 {
     #[apply(cached_lambda)]
     #[key(circ.info().hash, HashBytes)]
-    fn recurse(circ: CircuitRc) -> Result<CircuitRc, CircuitConstructionError> {
+    #[use_try]
+    fn recurse(circ: CircuitRc) -> Result<CircuitRc> {
         f(circ)?.map_children(&mut recurse).map(|z| z.rc())
     }
     recurse(circuit)
@@ -1470,9 +1299,9 @@ where
     }
 }
 
-pub fn visit_circuit_with_parents_fallible<F, E>(circuit: CircuitRc, mut f: F) -> Result<(), E>
+pub fn visit_circuit_with_parents_fallible<F>(circuit: CircuitRc, mut f: F) -> Result<()>
 where
-    F: FnMut(CircuitRc, &Vec<CircuitRc>) -> Result<(), E>,
+    F: FnMut(CircuitRc, &Vec<CircuitRc>) -> Result<()>,
 {
     let mut toposorted = toposort_circuit(circuit);
     toposorted.reverse(); // its children first by default
@@ -1492,20 +1321,16 @@ where
 /// because it's semantically cleaner to not have to think about which children are first
 #[apply(pycallable)]
 #[pyo3(name = "visit_circuit")]
-pub fn visit_circuit<F>(circuit: CircuitRc, mut f: F) -> Result<(), CircuitConstructionError>
+pub fn visit_circuit<F>(circuit: CircuitRc, mut f: F) -> Result<()>
 where
-    F: FnMut((circuit, CircuitRc)) -> Result<(), _>,
+    F: FnMut((circuit, CircuitRc)) -> Result<()>,
 {
     let mut f = f;
     let mut seen: HashSet<HashBytes> = HashSet::default();
 
-    fn recurse<F, CircuitConstructionError>(
-        circ: CircuitRc,
-        seen: &mut HashSet<HashBytes>,
-        f: &mut F,
-    ) -> Result<(), CircuitConstructionError>
+    fn recurse<F>(circ: CircuitRc, seen: &mut HashSet<HashBytes>, f: &mut F) -> Result<()>
     where
-        F: FnMut(CircuitRc) -> Result<(), CircuitConstructionError>,
+        F: FnMut(CircuitRc) -> Result<()>,
     {
         if !seen.contains(&circ.info().hash) {
             seen.insert(circ.info().hash);
@@ -1597,19 +1422,16 @@ where
     recurse(circuit)
 }
 
-pub fn deep_map_fallible_pre_new_children<F>(
-    circuit: CircuitRc,
-    f: F,
-) -> Result<CircuitRc, CircuitConstructionError>
+pub fn deep_map_fallible_pre_new_children<F>(circuit: CircuitRc, f: F) -> Result<CircuitRc>
 where
-    F: Fn(CircuitRc, &Vec<CircuitRc>) -> Result<CircuitRc, CircuitConstructionError>,
+    F: Fn(CircuitRc, &Vec<CircuitRc>) -> Result<CircuitRc>,
 {
     #[apply(cached_lambda)]
     #[key(circ.info().hash, HashBytes)]
-    fn recurse(circ: CircuitRc) -> Result<CircuitRc, CircuitConstructionError> {
+    #[use_try]
+    fn recurse(circ: CircuitRc) -> Result<CircuitRc> {
         let old_children: Vec<CircuitRc> = circ.children().collect(); // need to define this for borrow reasons
-        let new_children: Result<Vec<CircuitRc>, CircuitConstructionError> =
-            old_children.into_iter().map(recurse).collect();
+        let new_children: Result<Vec<CircuitRc>> = old_children.into_iter().map(recurse).collect();
         new_children.and_then(|a| f(circ, &a))
     }
     recurse(circuit)
@@ -1688,11 +1510,8 @@ where
     result
 }
 
-pub fn evaluate_fn_uncached(
-    circ: CircuitRc,
-    device_dtype: &TorchDeviceDtype,
-) -> Result<Tensor, TensorEvalError> {
-    let child_tensors: Result<Vec<Tensor>, TensorEvalError> = circ
+pub fn evaluate_fn_uncached(circ: CircuitRc, device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
+    let child_tensors: Result<Vec<Tensor>> = circ
         .children()
         .map(|x| evaluate_fn_uncached(x, device_dtype))
         .collect();
@@ -1753,7 +1572,7 @@ impl DerefMut for CircuitRc {
 }
 
 impl CircuitNodeInit for CircuitRc {
-    fn init_info_impl(self) -> Result<Self, CircuitConstructionError> {
+    fn init_info_impl(self) -> Result<Self> {
         Ok(self.c().clone().init_info()?.rc())
     }
 
@@ -1761,7 +1580,7 @@ impl CircuitNodeInit for CircuitRc {
         self.c().clone().rename(new_name).rc()
     }
 
-    fn update_info_impl<F>(self, f: F) -> Result<Self, CircuitConstructionError>
+    fn update_info_impl<F>(self, f: F) -> Result<Self>
     where
         F: FnOnce(&mut CachedCircuitInfo),
     {
@@ -1774,11 +1593,10 @@ impl CircuitNodeDefer for CircuitRc {
         // deref to avoid infinite recursion
         (**self).as_trait_obj()
     }
-    fn map_children_enumerate_impl<F, E>(&self, f: F) -> Result<Self, CircuitConstructionError>
+    fn map_children_enumerate_impl<F>(&self, f: F) -> Result<Self>
     where
         Self: Sized,
-        CircuitConstructionError: From<E>,
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc, E>,
+        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
     {
         (**self).map_children_enumerate(f).map(CircuitNode::rc)
     }
@@ -1928,18 +1746,17 @@ impl PyCircuitBase {
     }
 
     #[args(device_dtype = "Default::default()")]
-    fn evaluate(&self, device_dtype: TorchDeviceDtypeOp) -> Result<Tensor, TensorEvalError> {
+    fn evaluate(&self, device_dtype: TorchDeviceDtypeOp) -> Result<Tensor> {
         evaluate_fn_dtype_device(self.0.clone(), device_dtype)
     }
 
-    fn map_children_enumerate(&self, f: PyObject) -> Result<CircuitRc, CircuitConstructionError> {
+    fn map_children_enumerate(&self, f: PyObject) -> Result<CircuitRc> {
         self.0
-            .map_children_enumerate(|i, child| pycall!(f, (i, child), CircuitConstructionError))
+            .map_children_enumerate(|i, child| pycall!(f, (i, child), anyhow))
     }
 
-    fn map_children(&self, f: PyObject) -> Result<CircuitRc, CircuitConstructionError> {
-        self.0
-            .map_children(|child| pycall!(f, (child,), CircuitConstructionError))
+    fn map_children(&self, f: PyObject) -> Result<CircuitRc> {
+        self.0.map_children(|child| pycall!(f, (child,), anyhow))
     }
 
     fn total_arrayconstant_size(&self) -> BigUint {
@@ -1957,7 +1774,7 @@ impl PyCircuitBase {
         self.0.clone().add_suffix(suffix)
     }
 
-    fn visit(&self, f: PyObject) -> Result<(), CircuitConstructionError> {
+    fn visit(&self, f: PyObject) -> Result<()> {
         visit_circuit_py(self.0.clone(), f)
     }
 
@@ -1966,64 +1783,44 @@ impl PyCircuitBase {
         op_name: String,
         axis: Option<PyReductionAxes>,
         name: Option<String>,
-    ) -> Result<Circuit, CircuitConstructionError> {
+    ) -> Result<Circuit> {
         self.0
             .reduce(op_name, &reduction_to_ints(axis, self.info().rank()), name)
     }
 
     // sadly, below can't be defined with macro due to outer proc macro...
-    fn sum(
-        &self,
-        axis: Option<PyReductionAxes>,
-        name: Option<String>,
-    ) -> Result<CircuitRc, CircuitConstructionError> {
+    fn sum(&self, axis: Option<PyReductionAxes>, name: Option<String>) -> Result<CircuitRc> {
         Ok(self
             .0
             .sum(&reduction_to_ints(axis, self.info().rank()), name)?
             .rc())
     }
-    fn mean(
-        &self,
-        axis: Option<PyReductionAxes>,
-        name: Option<String>,
-    ) -> Result<CircuitRc, CircuitConstructionError> {
+    fn mean(&self, axis: Option<PyReductionAxes>, name: Option<String>) -> Result<CircuitRc> {
         Ok(self
             .0
             .mean(&reduction_to_ints(axis, self.info().rank()), name)?
             .rc())
     }
-    fn min(
-        &self,
-        axis: Option<PyReductionAxes>,
-        name: Option<String>,
-    ) -> Result<CircuitRc, CircuitConstructionError> {
+    fn min(&self, axis: Option<PyReductionAxes>, name: Option<String>) -> Result<CircuitRc> {
         Ok(self
             .0
             .min_(&reduction_to_ints(axis, self.info().rank()), name)?
             .rc())
     }
-    fn max(
-        &self,
-        axis: Option<PyReductionAxes>,
-        name: Option<String>,
-    ) -> Result<CircuitRc, CircuitConstructionError> {
+    fn max(&self, axis: Option<PyReductionAxes>, name: Option<String>) -> Result<CircuitRc> {
         Ok(self
             .0
             .max_(&reduction_to_ints(axis, self.info().rank()), name)?
             .rc())
     }
 
-    fn add(&self, other: CircuitRc, name: Option<String>) -> Result<Add, CircuitConstructionError> {
+    fn add(&self, other: CircuitRc, name: Option<String>) -> Result<Add> {
         self.0.add(other, name)
     }
-    fn sub(&self, other: CircuitRc, name: Option<String>) -> Result<Add, CircuitConstructionError> {
+    fn sub(&self, other: CircuitRc, name: Option<String>) -> Result<Add> {
         self.0.sub(other, name)
     }
-    fn mul(
-        &self,
-        other: CircuitRc,
-        name: Option<String>,
-    ) -> Result<Einsum, CircuitConstructionError> {
+    fn mul(&self, other: CircuitRc, name: Option<String>) -> Result<Einsum> {
         self.0.mul(other, name)
     }
     fn mul_scalar(
@@ -2031,14 +1828,10 @@ impl PyCircuitBase {
         scalar: f64,
         name: Option<String>,
         scalar_name: Option<String>,
-    ) -> Result<Einsum, CircuitConstructionError> {
+    ) -> Result<Einsum> {
         self.0.mul_scalar(scalar, name, scalar_name)
     }
-    fn index(
-        &self,
-        index: TensorIndex,
-        name: Option<String>,
-    ) -> Result<Index, CircuitConstructionError> {
+    fn index(&self, index: TensorIndex, name: Option<String>) -> Result<Index> {
         self.0.index(index, name)
     }
 }
@@ -2049,7 +1842,6 @@ pub fn get_compatible_dtype(circ: &Circuit) -> TorchDeviceDtype {
 
 pub mod prelude {
     pub use super::{
-        Circuit, CircuitConstructionError, CircuitNode, CircuitNodeAutoName, CircuitNodeUnion,
-        CircuitRc,
+        Circuit, CircuitNode, CircuitNodeAutoName, CircuitNodeUnion, CircuitRc, ConstructError,
     };
 }
