@@ -1,253 +1,24 @@
-use crate::all_imports::{TensorAxisIndex, TensorIndex};
-use crate::circuit::deep_map_fallible_pre_new_children;
-use rustc_hash::FxHashMap as HashMap;
-
-use crate::{pycall, pyo3_prelude::*, sv};
-use once_cell::sync::Lazy;
-use regex::Regex;
 use std::iter::zip;
 
-use super::circuit_manipulation::replace_nodes;
-
-use super::{
-    deep_map_op_context, deep_map_op_context_preorder_stoppable, deep_map_unwrap, prelude::*, Add,
-    CachedCircuitInfo, CircResult, Concat, GeneralFunction, HashBytes, Index, Rearrange, Scatter,
-};
-
-use crate::{
-    circuit::{Einsum, PyCircuitBase, Symbol},
-    circuit_node_auto_impl, circuit_node_extra_impl, new_rc_unwrap,
-    tensor_util::{Shape, TorchDeviceDtype},
-};
-
+use anyhow::{bail, Context, Result};
 use macro_rules_attribute::apply;
+use once_cell::sync::Lazy;
 use pyo3::types::PyDict;
-
+use regex::Regex;
+use rustc_hash::FxHashMap as HashMap;
 use uuid::{uuid, Uuid};
 
-#[pyfunction]
-#[pyo3(name = "expand_node")]
-pub fn expand_node_py(circuit: CircuitRc, inputs: Vec<CircuitRc>) -> CircResult {
-    expand_node(circuit, &inputs)
-}
-
-pub fn expand_node(circuit: CircuitRc, inputs: &Vec<CircuitRc>) -> CircResult {
-    if inputs.len() != circuit.children().count() {
-        return Err(CircuitConstructionError::ModuleNodeWrongNumberChildren {
-            expected: circuit.children().count(),
-            got: inputs.len(),
-        });
-    }
-    let batch_ranks: Vec<usize> = zip(circuit.children(), inputs)
-        .filter_map(|(old, new)| new.info().rank().checked_sub(old.info().rank()))
-        .collect();
-    if batch_ranks.len() != inputs.len() {
-        return Err(CircuitConstructionError::BatchingRankTooLow {
-            default: circuit.children().map(|x| x.info().rank()).collect(),
-            got: inputs.iter().map(|x| x.info().rank()).collect(),
-        });
-    }
-    let batch_shapes: Vec<&[usize]> = zip(&batch_ranks, inputs)
-        .map(|(br, new)| &new.info().shape[0..*br])
-        .collect();
-    match &**circuit {
-        Circuit::Symbol(_) | Circuit::ScalarConstant(_) => Ok(circuit.clone()),
-        Circuit::Rearrange(node) => {
-            let input_shape_non_batch = inputs[0].info().shape[batch_ranks[0]..]
-                .iter()
-                .cloned()
-                .collect();
-            let new_spec = node
-                .spec
-                .conform_to_input_shape(&input_shape_non_batch, true)
-                .map_err(|_e| CircuitConstructionError::RearrangeWrongInputShape {
-                    spec: node.spec.clone(),
-                    shape: input_shape_non_batch.clone(),
-                })?
-                .add_batch_dims(batch_ranks[0]);
-            Ok(Rearrange::nrc(
-                inputs[0].clone(),
-                new_spec,
-                circuit.name_cloned(),
-            ))
-        }
-        Circuit::Index(node) => {
-            // for now non-batch non-identity dims can't change
-            for i in 0..node.node.info().rank() {
-                if node.node.info().shape[i] != inputs[0].info().shape[i + batch_ranks[0]]
-                    && node.index.0[i] != TensorAxisIndex::IDENT
-                {
-                    return Err(CircuitConstructionError::ExpandingFixedIndex {
-                        index: node.index.clone(),
-                        old_shape: node.node.info().shape.clone(),
-                        new_shape: inputs[0].info().shape.clone(),
-                    });
-                }
-            }
-            Ok(Index::nrc(
-                inputs[0].clone(),
-                TensorIndex(
-                    vec![TensorAxisIndex::IDENT; batch_ranks[0]]
-                        .into_iter()
-                        .chain(node.index.0.iter().cloned())
-                        .collect(),
-                ),
-                node.name_cloned(),
-            ))
-        }
-        Circuit::Scatter(node) => {
-            // for now non-batch non-identity dims can't change
-            for i in 0..node.node.info().rank() {
-                if node.node.info().shape[i] != inputs[0].info().shape[i + batch_ranks[0]]
-                    && node.index.0[i] != TensorAxisIndex::IDENT
-                {
-                    return Err(CircuitConstructionError::ExpandingFixedIndex {
-                        index: node.index.clone(),
-                        old_shape: node.node.info().shape.clone(),
-                        new_shape: inputs[0].info().shape.clone(),
-                    });
-                }
-            }
-            Ok(Scatter::nrc(
-                inputs[0].clone(),
-                TensorIndex(
-                    vec![TensorAxisIndex::IDENT; batch_ranks[0]]
-                        .into_iter()
-                        .chain(node.index.0.iter().cloned())
-                        .collect(),
-                ),
-                inputs[0].info().shape[0..batch_ranks[0]]
-                    .iter()
-                    .cloned()
-                    .chain(node.info().shape.iter().cloned())
-                    .collect(),
-                node.name_cloned(),
-            ))
-        }
-        Circuit::Concat(node) => {
-            if !batch_shapes.iter().all(|x| x == &batch_shapes[0]) {
-                return Err(CircuitConstructionError::InconsistentBatches {
-                    batch_shapes: batch_shapes
-                        .iter()
-                        .map(|x| x.iter().cloned().collect())
-                        .collect(),
-                    circuit: circuit.clone(),
-                });
-            }
-            if !zip(&node.nodes, zip(inputs, &batch_ranks)).all(|(old, (new, br))| {
-                old.info().shape[node.axis] == new.info().shape[node.axis + br]
-            }) {
-                return Err(CircuitConstructionError::ExpandingConcatAxis {
-                    axis: node.axis,
-                    old_shape: sv![],
-                    new_shape: sv![],
-                });
-            }
-            Concat::try_new(
-                inputs.clone(),
-                node.axis + batch_ranks[0],
-                node.name_cloned(),
-            )
-            .map(|x| x.rc())
-        }
-        Circuit::Add(node) => Add::try_new(inputs.clone(), node.name_cloned()).map(|x| x.rc()),
-        Circuit::GeneralFunction(node) => {
-            GeneralFunction::try_new(inputs.clone(), node.spec.clone(), node.name_cloned())
-                .map(|x| x.rc())
-        }
-        Circuit::Einsum(node) => {
-            let mut batch_shape: Option<&[usize]> = None;
-            for bs in &batch_shapes {
-                if !bs.is_empty() {
-                    if let Some(existing) = batch_shape {
-                        if *bs != existing {
-                            return Err(CircuitConstructionError::InconsistentBatches {
-                                batch_shapes: batch_shapes
-                                    .iter()
-                                    .map(|x| x.iter().cloned().collect())
-                                    .collect(),
-                                circuit: circuit.clone(),
-                            });
-                        }
-                    } else {
-                        batch_shape = Some(bs.clone());
-                    }
-                }
-            }
-            let next_axis = node.next_axis();
-            let newies = || (next_axis as u8..next_axis + batch_shape.unwrap().len() as u8);
-            let out_axes = if let Some(_bs) = batch_shape {
-                newies().chain(node.out_axes.iter().cloned()).collect()
-            } else {
-                node.out_axes.clone()
-            };
-            Einsum::try_new(
-                node.args
-                    .iter()
-                    .enumerate()
-                    .map(|(i, (_child, ints))| {
-                        (inputs[i].clone(), {
-                            if !batch_shapes[i].is_empty() {
-                                newies().chain(ints.iter().cloned()).collect()
-                            } else {
-                                ints.clone()
-                            }
-                        })
-                    })
-                    .collect(),
-                out_axes,
-                node.name_cloned(),
-            )
-            .map(|x| x.rc())
-        }
-        Circuit::ModuleNode(node) => {
-            ModuleNode::try_new(inputs.clone(), node.spec.clone(), node.name_cloned())
-                .map(|z| z.rc())
-        }
-        _ => {
-            if inputs[..] == circuit.children().collect::<Vec<_>>()[..] {
-                Ok(circuit.clone())
-            } else {
-                Err(CircuitConstructionError::ExpandNodeUnknownVariant {
-                    variant: circuit.variant_string(),
-                })
-            }
-        }
-    }
-}
-
-#[pyfunction]
-#[pyo3(name = "replace_expand_bottom_up_dict")]
-pub fn replace_expand_bottom_up_dict_py(
-    circuit: CircuitRc,
-    dict: HashMap<CircuitRc, CircuitRc>,
-) -> CircResult {
-    replace_expand_bottom_up(circuit, |x| dict.get(&x).cloned())
-}
-
-#[pyfunction]
-#[pyo3(name = "replace_expand_bottom_up")]
-pub fn replace_expand_bottom_up_py(circuit: CircuitRc, f: PyObject) -> CircResult {
-    replace_expand_bottom_up(circuit, |x| pycall!(f, (x.clone(),)))
-}
-
-pub fn replace_expand_bottom_up<F>(
-    circuit: CircuitRc,
-    replacer: F,
-) -> Result<CircuitRc, CircuitConstructionError>
-where
-    F: Fn(CircuitRc) -> Option<CircuitRc>,
-{
-    let recursor = |circuit: CircuitRc,
-                    new_children: &Vec<CircuitRc>|
-     -> Result<CircuitRc, CircuitConstructionError> {
-        if let Some(replaced) = replacer(circuit.clone()) {
-            return Ok(replaced);
-        }
-        expand_node(circuit, new_children)
-    };
-    deep_map_fallible_pre_new_children(circuit, recursor)
-}
+use super::{
+    circuit_manipulation::replace_nodes, deep_map_op_context,
+    deep_map_op_context_preorder_stoppable, deep_map_unwrap, expand_node::replace_expand_bottom_up,
+    prelude::*, CachedCircuitInfo, HashBytes,
+};
+use crate::{
+    circuit::{expand_node::ExpandError, PyCircuitBase, Symbol},
+    circuit_node_auto_impl, circuit_node_extra_impl, new_rc_unwrap,
+    pyo3_prelude::*,
+    tensor_util::{Shape, TorchDeviceDtype},
+};
 
 #[pyclass]
 #[derive(Debug, Clone, PyClassDeriv, Hash, PartialEq, Eq)]
@@ -262,9 +33,9 @@ pub struct ModuleNodeSpec {
 
 impl ModuleNodeSpec {
     pub const EXPAND_PLACEHOLDER_UUID: Uuid = uuid!("741ba404-eec3-4ac9-b6ce-062e903fb033");
-    pub fn expand_raw(&self, nodes: &Vec<CircuitRc>) -> CircResult {
+    pub fn expand_raw(&self, nodes: &Vec<CircuitRc>) -> Result<CircuitRc> {
         if self.input_specs.len() != nodes.len() {
-            return Err(CircuitConstructionError::ModuleNodeWrongNumberChildren {
+            bail!(ConstructError::ModuleNodeWrongNumberChildren {
                 expected: self.input_specs.len(),
                 got: nodes.len(),
             });
@@ -280,7 +51,7 @@ impl ModuleNodeSpec {
                         .collect::<Vec<_>>()
                 );
                 self.spec_circuit.compiler_print();
-                return Err(CircuitConstructionError::InconsistentBatches {
+                bail!(ExpandError::InconsistentBatches {
                     batch_shapes: vec![spec.symbol.info().shape.clone(), node.info().shape.clone()],
                     circuit: self.spec_circuit.clone(),
                 });
@@ -294,7 +65,7 @@ impl ModuleNodeSpec {
                         .collect::<Vec<_>>()
                 );
                 self.spec_circuit.compiler_print();
-                return Err(CircuitConstructionError::InconsistentBatches {
+                bail!(ExpandError::InconsistentBatches {
                     batch_shapes: vec![spec.symbol.info().shape.clone(), node.info().shape.clone()],
                     circuit: self.spec_circuit.clone(),
                 });
@@ -311,7 +82,7 @@ impl ModuleNodeSpec {
                         .collect::<Vec<_>>()
                 );
                 self.spec_circuit.compiler_print();
-                return Err(CircuitConstructionError::InconsistentBatches {
+                bail!(ExpandError::InconsistentBatches {
                     batch_shapes: vec![spec.symbol.info().shape.clone(), node.info().shape.clone()],
                     circuit: self.spec_circuit.clone(),
                 });
@@ -325,11 +96,11 @@ impl ModuleNodeSpec {
         })
     }
 
-    pub fn expand_shape(&self, shapes: &Vec<Shape>) -> CircResult {
+    pub fn expand_shape(&self, shapes: &Vec<Shape>) -> Result<CircuitRc> {
         if let Some(result) = MODULE_EXPANSIONS_SHAPE
             .with(|cache| cache.borrow().get(&(self.clone(), shapes.clone())).cloned())
         {
-            return result;
+            return Ok(result);
         }
         let symbols = shapes
             .iter()
@@ -342,47 +113,46 @@ impl ModuleNodeSpec {
                 )
             })
             .collect();
-        let result = self.expand_raw(&symbols);
+        let result = self.expand_raw(&symbols)?;
         MODULE_EXPANSIONS_SHAPE.with(|cache| {
             cache
                 .borrow_mut()
                 .insert((self.clone(), shapes.clone()), result.clone())
         });
-        result
+        Ok(result)
     }
 
-    pub fn expand(&self, nodes: &Vec<CircuitRc>) -> CircResult {
+    pub fn expand(&self, nodes: &Vec<CircuitRc>) -> Result<CircuitRc> {
         let key: (ModuleNodeSpec, Vec<HashBytes>) =
             (self.clone(), nodes.iter().map(|x| x.info().hash).collect());
-        MODULE_EXPANSIONS
-            .with(|cache| cache.borrow().get(&key).cloned())
-            .unwrap_or_else(|| {
-                let result = (|| {
-                    let shapes = nodes.iter().map(|x| x.info().shape.clone()).collect();
-                    let expanded_shape = self.expand_shape(&shapes)?;
-                    Ok(replace_nodes(
-                        expanded_shape,
-                        &nodes
-                            .iter()
-                            .enumerate()
-                            .map(|(i, n)| {
-                                (
-                                    Symbol::new(
-                                        n.info().shape.clone(),
-                                        ModuleNodeSpec::EXPAND_PLACEHOLDER_UUID,
-                                        Some(format!("{}_{:?}", i, n.info().shape)),
-                                    )
-                                    .info()
-                                    .hash,
-                                    n.clone(),
-                                )
-                            })
-                            .collect(),
-                    ))
-                })();
-                MODULE_EXPANSIONS.with(|cache| cache.borrow_mut().insert(key, result.clone()));
-                result
-            })
+
+        if let Some(result) = MODULE_EXPANSIONS.with(|cache| cache.borrow().get(&key).cloned()) {
+            return Ok(result);
+        }
+
+        let shapes = nodes.iter().map(|x| x.info().shape.clone()).collect();
+        let expanded_shape = self.expand_shape(&shapes)?;
+        let result = replace_nodes(
+            expanded_shape,
+            &nodes
+                .iter()
+                .enumerate()
+                .map(|(i, n)| {
+                    (
+                        Symbol::new(
+                            n.info().shape.clone(),
+                            ModuleNodeSpec::EXPAND_PLACEHOLDER_UUID,
+                            Some(format!("{}_{:?}", i, n.info().shape)),
+                        )
+                        .info()
+                        .hash,
+                        n.clone(),
+                    )
+                })
+                .collect(),
+        );
+        MODULE_EXPANSIONS.with(|cache| cache.borrow_mut().insert(key, result.clone()));
+        Ok(result)
     }
 
     pub fn compute_hash(&self) -> HashBytes {
@@ -411,10 +181,7 @@ impl ModuleNodeSpec {
     }
 
     #[staticmethod]
-    pub fn new_auto(
-        circuit: CircuitRc,
-        name: Option<String>,
-    ) -> Result<Self, CircuitConstructionError> {
+    pub fn new_auto(circuit: CircuitRc, name: Option<String>) -> Result<Self> {
         let mut input_specs_dict: HashMap<usize, ModuleNodeArgSpec> = HashMap::default();
         let circuit = deep_map_op_context(
             circuit.clone(),
@@ -508,7 +275,7 @@ impl ModuleNodeSpec {
         })
     }
 
-    pub fn resize(&self, shapes: Vec<Shape>) -> Result<Self, CircuitConstructionError> {
+    pub fn resize(&self, shapes: Vec<Shape>) -> Result<Self> {
         let input_specs: Vec<ModuleNodeArgSpec> = zip(&self.input_specs, shapes)
             .map(|(spec, shape)| ModuleNodeArgSpec {
                 symbol: Symbol::new(shape, spec.symbol.uuid, spec.symbol.name_cloned()),
@@ -570,10 +337,16 @@ impl ModuleNode {
         nodes: Vec<CircuitRc>,
         spec: ModuleNodeSpec,
         name: Option<String>,
-    ) -> (Result<Self, CircuitConstructionError>) {
+    ) -> Result<Self> {
         let shape = spec
             .expand_shape(&nodes.iter().map(|x| x.info().shape.clone()).collect())
-            .map_err(|e| CircuitConstructionError::ModuleNodeExpansionError { error: Box::new(e) })?
+            .context(format!(
+                "module node expansion error{}",
+                spec.name
+                    .as_ref()
+                    .map(|s| format!(" ({})", s))
+                    .unwrap_or(String::new())
+            ))?
             .info()
             .shape
             .clone();
@@ -593,7 +366,7 @@ impl ModuleNode {
         kwargs: &HashMap<String, CircuitRc>,
         spec: ModuleNodeSpec,
         name: Option<String>,
-    ) -> Result<Self, CircuitConstructionError> {
+    ) -> Result<Self> {
         let mut nodes: Vec<CircuitRc> = vec![spec.spec_circuit.clone(); spec.input_specs.len()];
         for (k, v) in kwargs {
             match spec
@@ -605,7 +378,7 @@ impl ModuleNode {
                     nodes[i] = v.clone();
                 }
                 None => {
-                    return Err(CircuitConstructionError::ModuleNodeUnknownArgument {
+                    bail!(ConstructError::ModuleNodeUnknownArgument {
                         argument: k.clone(),
                     })
                 }
@@ -638,10 +411,9 @@ impl CircuitNode for ModuleNode {
         Box::new(self.nodes.iter().cloned())
     }
 
-    fn map_children_enumerate<F, E>(&self, mut f: F) -> Result<Self, CircuitConstructionError>
+    fn map_children_enumerate<F>(&self, mut f: F) -> Result<Self>
     where
-        CircuitConstructionError: From<E>,
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc, E>,
+        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
     {
         Self::try_new(
             self.nodes
@@ -662,7 +434,7 @@ impl CircuitNode for ModuleNode {
         &self,
         _tensors: &[crate::py_types::Tensor],
         _device_dtype: &TorchDeviceDtype,
-    ) -> Result<crate::py_types::Tensor, super::TensorEvalError> {
+    ) -> Result<crate::py_types::Tensor> {
         unimplemented!();
     }
 }
@@ -710,19 +482,14 @@ impl ModuleNode {
     }
 
     #[staticmethod]
-    fn new_flat(
-        nodes: Vec<CircuitRc>,
-        spec: ModuleNodeSpec,
-        name: Option<String>,
-    ) -> Result<Self, CircuitConstructionError> {
+    fn new_flat(nodes: Vec<CircuitRc>, spec: ModuleNodeSpec, name: Option<String>) -> Result<Self> {
         Self::try_new(nodes, spec, name)
     }
 
     pub fn expand(&self) -> CircuitRc {
         self.spec
             .expand(&self.nodes)
-            .map_err(|e| CircuitConstructionError::ModuleNodeExpansionError { error: Box::new(e) })
-            .unwrap()
+            .expect("module expansion fail!") // maybe this is supposed to return error instead of panicking?
     }
 }
 
@@ -735,8 +502,8 @@ pub fn inline_all_modules(circuit: CircuitRc) -> CircuitRc {
 }
 use std::cell::RefCell;
 thread_local! {
-    static MODULE_EXPANSIONS: RefCell<HashMap<(ModuleNodeSpec, Vec<HashBytes>), CircResult>> =
+    static MODULE_EXPANSIONS: RefCell<HashMap<(ModuleNodeSpec, Vec<HashBytes>), CircuitRc>> =
         RefCell::new(HashMap::default());
-    static MODULE_EXPANSIONS_SHAPE: RefCell<HashMap<(ModuleNodeSpec, Vec<Shape>), CircResult>> =
+    static MODULE_EXPANSIONS_SHAPE: RefCell<HashMap<(ModuleNodeSpec, Vec<Shape>), CircuitRc>> =
         RefCell::new(HashMap::default());
 }
