@@ -1,13 +1,14 @@
-use crate::util::is_unique;
-use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
 use std::collections::BTreeMap;
 
+use anyhow::Result;
+use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
 use uuid::Uuid;
 
-use super::circuit_utils::deep_map_pass_down_branching;
-use super::{deep_map_unwrap, prelude::*, visit_circuit_with_parents_fallible, NamedAxes};
-
-use crate::pyo3_prelude::*;
+use super::{
+    circuit_utils::deep_map_pass_down_branching, deep_map_unwrap, prelude::*,
+    visit_circuit_with_parents_fallible, NamedAxes,
+};
+use crate::{pyo3_prelude::*, util::is_unique};
 
 pub fn set_named_axes<T: CircuitNode>(node: T, named_axes: NamedAxes) -> T {
     node.update_info(|i| i.named_axes = named_axes).unwrap()
@@ -96,15 +97,12 @@ pub fn get_axis_leaves(
     circuit: CircuitRc,
     axis: usize,
     _error_on_removable_axis_leaf: bool,
-) -> Result<
-    (
-        String,
-        CircuitRc,
-        HashMap<CircuitRc, usize>,
-        HashSet<CircuitRc>,
-    ),
-    CircuitConstructionError,
-> {
+) -> Result<(
+    String,
+    CircuitRc,
+    HashMap<CircuitRc, usize>,
+    HashSet<CircuitRc>,
+)> {
     let name = Uuid::new_v4().to_string();
     if circuit
         .as_einsum()
@@ -152,7 +150,7 @@ pub fn get_axis_leaves(
         {
             result_no_axis.insert(x);
         }
-        Ok::<(), CircuitConstructionError>(())
+        Ok(())
     })?;
     Ok((name, circ_named_axes, result_axis, result_no_axis))
 }
