@@ -12,9 +12,9 @@ macro_rules! new_rc {
 }
 #[macro_export]
 macro_rules! new_rc_unwrap {
-    ($pub:vis fn $fname:ident($( $arg:ident : $argtype:ty),* $(,)?) -> (Result<$rettype:ty,CircuitConstructionError>)  $body:expr)=>{
-        $pub fn $fname($($arg : $argtype),*)->Result<$rettype,CircuitConstructionError> {
-            $body
+    ($pub:vis fn $fname:ident($( $arg:ident : $argtype:ty),* $(,)?) -> Result<$rettype:ty>  { $($body:tt)* }) =>{
+        $pub fn $fname($($arg : $argtype),*)->anyhow::Result<$rettype> {
+            $($body)*
         }
 
         pub fn nrc($($arg : $argtype),*)->CircuitRc{
