@@ -1,24 +1,27 @@
-use super::{
-    Add, ArrayConstant, Concat, Einsum, GeneralFunction, Index, Rearrange, ScalarConstant, Scatter,
-    Symbol,
-};
-use super::{CircuitConstructionError, CircuitRc};
-use crate::all_imports::{Shape, Sv, TensorIndex};
-use crate::circuit::{
-    AutoTag, CircResult, CircuitNode, Cumulant, DiscreteVar, ModuleNode, ModuleNodeSpec,
-    StoredCumulantVar,
-};
-use crate::lru_cache::TensorCacheRrfs;
-use crate::pyo3_prelude::*;
-use crate::tensor_util::TorchDeviceDtypeOp;
+use std::{iter::zip, str::FromStr};
+
+use anyhow::{bail, Context, Result};
+use macro_rules_attribute::apply;
 use once_cell::sync::Lazy;
+use pyo3::exceptions::PyValueError;
 use regex::Regex;
 use rustc_hash::FxHashMap as HashMap;
-use std::iter::zip;
-use std::str::FromStr;
-use std::sync::Arc;
+use thiserror::Error;
 use uuid::Uuid;
 
+use super::{
+    Add, ArrayConstant, AutoTag, CircuitNode, CircuitRc, Concat, Cumulant, DiscreteVar, Einsum,
+    GeneralFunction, Index, ModuleNode, ModuleNodeSpec, Rearrange, ScalarConstant, Scatter,
+    StoredCumulantVar, Symbol,
+};
+use crate::{
+    all_imports::{Shape, Sv, TensorIndex},
+    lru_cache::TensorCacheRrfs,
+    pyo3_prelude::*,
+    python_error_exception,
+    tensor_util::TorchDeviceDtypeOp,
+};
+
 #[pyfunction(
     string,
     module_spec_map = "HashMap::default()",
@@ -35,7 +38,7 @@ pub fn parse_compiler_repr_bijection_py(
     tensors_as_random: bool,
     tensors_as_random_device_dtype: TorchDeviceDtypeOp,
     tensor_cache: Option<TensorCacheRrfs>,
-) -> Result<CircuitRc, CircuitConstructionError> {
+) -> Result<CircuitRc> {
     let mut tensor_cache = tensor_cache;
     parse_compiler_repr_bijection(
         string,
@@ -54,7 +57,7 @@ pub fn parse_compiler_repr_bijection(
     tensors_as_random: bool,
     tensors_as_random_device_dtype: TorchDeviceDtypeOp,
     tensor_cache: &mut Option<TensorCacheRrfs>,
-) -> Result<CircuitRc, CircuitConstructionError> {
+) -> Result<CircuitRc> {
     let string = string.trim();
     let lines: Vec<_> = string.lines().collect();
     // make seperate struct that can deeply mutate, can't use immutable Circuit bc see children later
@@ -73,30 +76,34 @@ pub fn parse_compiler_repr_bijection(
         Regex::new(r"^( *)(\d+)(?: '((?:(?:\\')?[^']*)*)')?(?: \[([\d, ]*)\])?(?: ([a-zA-Z]+))?(?: (.*))?\s*$").unwrap()
     });
     for line in lines {
-        let rf = || CircuitConstructionError::ParsingNoRegexMatch {
+        let rf = || ParseError::NoRegexMatch {
             line: line.to_owned(),
         };
         let re_captures = RE.captures(line).ok_or_else(rf)?;
         let num_spaces = re_captures.get(1).ok_or_else(rf)?.as_str().len();
         if num_spaces % tab_width != 0 {
-            return Err(CircuitConstructionError::ParsingInvalidIndentation {
-                tab_width: tab_width,
+            bail!(ParseError::InvalidIndentation {
+                tab_width,
                 spaces: num_spaces,
                 stack_indentation: stack.len(),
-                stack_top: stack
-                    .last()
-                    .map(|z| partial_circuits[*z].clone().unwrap().variant.clone()),
+                stack_top: stack.last().map(|z| partial_circuits[*z]
+                    .clone()
+                    .unwrap()
+                    .variant
+                    .clone()),
             });
         }
         let indentation_level = num_spaces / tab_width;
         if indentation_level > stack.len() {
-            return Err(CircuitConstructionError::ParsingInvalidIndentation {
-                tab_width: tab_width,
+            bail!(ParseError::InvalidIndentation {
+                tab_width,
                 spaces: num_spaces,
                 stack_indentation: stack.len(),
-                stack_top: stack
-                    .last()
-                    .map(|z| partial_circuits[*z].clone().unwrap().variant.clone()),
+                stack_top: stack.last().map(|z| partial_circuits[*z]
+                    .clone()
+                    .unwrap()
+                    .variant
+                    .clone()),
             });
         }
         stack.truncate(indentation_level);
@@ -127,7 +134,7 @@ pub fn parse_compiler_repr_bijection(
                             .filter(|z| !z.is_empty())
                             .map(|x| x.parse::<usize>())
                             .collect::<Result<Sv<_>, _>>()
-                            .map_err(|e| CircuitConstructionError::ParsingNumberFail {
+                            .map_err(|e| ParseError::NumberFail {
                                 string: format!("{}", e),
                             })?,
                     )
@@ -165,16 +172,16 @@ pub fn parse_compiler_repr_bijection(
         tensors_as_random: bool,
         tensors_as_random_device_dtype: TorchDeviceDtypeOp,
         tensor_cache: &mut Option<TensorCacheRrfs>,
-    ) -> CircResult {
+    ) -> Result<CircuitRc> {
         if let Some(already) = context.get(&serial_number) {
             return Ok(already.clone());
         }
         if serial_number > partial_circuits.len() - 1 {
-            return Err(CircuitConstructionError::ParsingInvalidSerialNumber { serial_number });
+            bail!(ParseError::InvalidSerialNumber { serial_number });
         }
         let ps = partial_circuits[serial_number]
             .as_ref()
-            .ok_or(CircuitConstructionError::ParsingInvalidSerialNumber { serial_number })?;
+            .ok_or(ParseError::InvalidSerialNumber { serial_number })?;
         let children: Vec<CircuitRc> = ps
             .children
             .iter()
@@ -195,180 +202,163 @@ pub fn parse_compiler_repr_bijection(
         let result = match variant {
             "Array" => {
                 if tensors_as_random {
-                    Ok(ArrayConstant::randn_named(
-                        ps.shape
-                            .clone()
-                            .ok_or(CircuitConstructionError::ParsingShapeNeeded {
-                                variant: variant.to_owned(),
-                            })?,
+                    ArrayConstant::randn_named(
+                        ps.shape.clone().ok_or(ParseError::ShapeNeeded {
+                            variant: variant.to_owned(),
+                        })?,
                         ps.name.clone(),
                         tensors_as_random_device_dtype,
                     )
-                    .rc())
+                    .rc()
                 } else {
                     ArrayConstant::from_hash_prefix(ps.name.clone(), &ps.extra, tensor_cache)
-                        .map(|z| z.rc())
-                        .map_err(|e| CircuitConstructionError::PythonError {
-                            py_err: Arc::new(e),
-                        })
+                        .context("parse array constant from hash prefix")?
+                        .rc()
                 }
             }
-            "Scalar" => Ok(ScalarConstant::nrc(
-                ps.extra.parse::<f64>().map_err(|e| {
-                    CircuitConstructionError::ParsingNumberFail {
+            "Scalar" => ScalarConstant::nrc(
+                ps.extra
+                    .parse::<f64>()
+                    .map_err(|e| ParseError::NumberFail {
                         string: format!("{}", e),
-                    }
-                })?,
+                    })?,
                 ps.shape
                     .as_ref()
-                    .ok_or(CircuitConstructionError::ParsingShapeNeeded {
+                    .ok_or(ParseError::ShapeNeeded {
                         variant: variant.to_owned(),
                     })?
                     .clone(),
                 ps.name.clone(),
-            )),
+            ),
             "Add" => {
                 if !ps.extra.is_empty() {
-                    Err(CircuitConstructionError::ParsingExtraUnneededString {
+                    bail!(ParseError::ExtraUnneededString {
                         string: ps.extra.to_owned(),
                     })
                 } else {
-                    Add::try_new(children, ps.name.clone()).map(|z| z.rc())
+                    Add::try_new(children, ps.name.clone())?.rc()
                 }
             }
             "Concat" => Concat::try_new(
                 children,
-                ps.extra.parse::<usize>().map_err(|e| {
-                    CircuitConstructionError::ParsingNumberFail {
+                ps.extra
+                    .parse::<usize>()
+                    .map_err(|e| ParseError::NumberFail {
                         string: format!("{}", e),
-                    }
-                })?,
+                    })?,
                 ps.name.clone(),
-            )
-            .map(|z| z.rc()),
-            "Einsum" => {
-                Einsum::from_einsum_string(&ps.extra, children, ps.name.clone()).map(|z| z.rc())
-            }
+            )?
+            .rc(),
+            "Einsum" => Einsum::from_einsum_string(&ps.extra, children, ps.name.clone())?.rc(),
             "Rearrange" => {
-                Rearrange::from_einops_string(children[0].clone(), &ps.extra, ps.name.clone())
-                    .map(|z| z.rc())
+                Rearrange::from_einops_string(children[0].clone(), &ps.extra, ps.name.clone())?.rc()
             }
             "Symbol" => {
                 let shape = ps
                     .shape
                     .as_ref()
-                    .ok_or(CircuitConstructionError::ParsingShapeNeeded {
+                    .ok_or(ParseError::ShapeNeeded {
                         variant: variant.to_owned(),
                     })?
                     .clone();
                 if ps.extra.is_empty() {
-                    Ok(Symbol::new_with_none_uuid(shape, ps.name.clone()).rc())
+                    Symbol::new_with_none_uuid(shape, ps.name.clone()).rc()
                 } else {
-                    Ok(Symbol::nrc(
+                    Symbol::nrc(
                         shape,
-                        Uuid::from_str(&ps.extra).map_err(|_e| {
-                            CircuitConstructionError::ParsingFail {
-                                string: ps.extra.to_owned(),
-                            }
+                        Uuid::from_str(&ps.extra).map_err(|_e| ParseError::Fail {
+                            string: ps.extra.to_owned(),
                         })?,
                         ps.name.clone(),
-                    ))
+                    )
                 }
             }
             "GeneralFunction" => {
-                GeneralFunction::new_by_name(children, ps.extra.clone(), ps.name.clone())
-                    .map(|z| z.rc())
+                GeneralFunction::new_by_name(children, ps.extra.clone(), ps.name.clone())?.rc()
             }
             "Index" => Index::try_new(
                 children[0].clone(),
                 TensorIndex::from_bijection_string(&ps.extra, tensor_cache)?,
                 ps.name.clone(),
-            )
-            .map(|z| z.rc()),
+            )?
+            .rc(),
             "Scatter" => Scatter::try_new(
                 children[0].clone(),
                 TensorIndex::from_bijection_string(&ps.extra, tensor_cache)?,
                 ps.shape
                     .as_ref()
-                    .ok_or(CircuitConstructionError::ParsingShapeNeeded {
+                    .ok_or(ParseError::ShapeNeeded {
                         variant: variant.to_owned(),
                     })?
                     .clone(),
                 ps.name.clone(),
-            )
-            .map(|z| z.rc()),
+            )?
+            .rc(),
             "ModuleNode" => ModuleNode::try_new(
                 children,
                 module_spec_map
                     .get(&ps.extra.clone())
                     .cloned()
-                    .ok_or_else(|| CircuitConstructionError::ModuleNotFound {
+                    .ok_or_else(|| ParseError::ModuleNotFound {
                         name: ps.extra.clone(),
                     })?,
                 ps.name.clone(),
-            )
-            .map(|z| z.rc()),
+            )?
+            .rc(),
             "Ref" => reference_circuits
                 .get(&ps.extra.clone())
                 .cloned()
-                .ok_or_else(|| CircuitConstructionError::CircuitRefNotFound {
+                .ok_or_else(|| ParseError::CircuitRefNotFound {
                     name: ps.extra.clone(),
-                }),
+                })?,
             "AutoTag" => Uuid::from_str(&ps.extra)
-                .map_err(|_| CircuitConstructionError::InvalidUuid {
+                .map_err(|_| ParseError::InvalidUuid {
                     string: ps.extra.clone(),
                 })
-                .map(|uuid| AutoTag::nrc(children[0].clone(), uuid, ps.name.clone())),
-            "Cumulant" => Ok(Cumulant::nrc(children, ps.name.clone())),
+                .map(|uuid| AutoTag::nrc(children[0].clone(), uuid, ps.name.clone()))?,
+            "Cumulant" => Cumulant::nrc(children, ps.name.clone()),
             "DiscreteVar" => {
-                DiscreteVar::try_new(children[0].clone(), children[1].clone(), ps.name.clone())
-                    .map(|z| z.rc())
+                DiscreteVar::try_new(children[0].clone(), children[1].clone(), ps.name.clone())?
+                    .rc()
             }
             "StoredCumulantVar" => {
                 if let [cum_nums, uuid] = ps.extra.split("|").collect::<Vec<_>>()[..] {
-                    cum_nums
+                    let keys = cum_nums
                         .split(",")
                         .map(|s| s.trim().parse::<usize>())
                         .collect::<Result<Vec<_>, _>>()
-                        .map_err(|e| CircuitConstructionError::ParsingNumberFail {
+                        .map_err(|e| ParseError::NumberFail {
                             string: format!("{}", e),
+                        })?;
+
+                    let uuid = Uuid::from_str(uuid).map_err(|_| ParseError::InvalidUuid {
+                        string: ps.extra.clone(),
+                    })?;
+                    if keys.len() != children.len() {
+                        bail!(ParseError::WrongNumberChildren {
+                            expected: keys.len(),
+                            found: children.len(),
                         })
-                        .and_then(|keys| {
-                            Uuid::from_str(uuid)
-                                .map_err(|_| CircuitConstructionError::InvalidUuid {
-                                    string: ps.extra.clone(),
-                                })
-                                .map(|uuid| (keys, uuid))
-                        })
-                        .and_then(|(keys, uuid)| {
-                            if keys.len() != children.len() {
-                                Err(CircuitConstructionError::ParsingWrongNumberChildren {
-                                    expected: keys.len(),
-                                    found: children.len(),
-                                })
-                            } else {
-                                StoredCumulantVar::try_new(
-                                    zip(keys, children).collect(),
-                                    uuid,
-                                    ps.name.clone(),
-                                )
-                                .map(|z| z.rc())
-                            }
-                        })
+                    } else {
+                        StoredCumulantVar::try_new(
+                            zip(keys, children).collect(),
+                            uuid,
+                            ps.name.clone(),
+                        )?
+                        .rc()
+                    }
                 } else {
-                    Err(CircuitConstructionError::ParsingFail {
+                    bail!(ParseError::Fail {
                         string: ps.extra.clone(),
                     })
                 }
             }
-            _ => Err(CircuitConstructionError::ParsingInvalidVariant {
+            _ => bail!(ParseError::InvalidVariant {
                 v: variant.to_owned(),
             }),
         };
-        if result.is_ok() {
-            context.insert(serial_number, result.clone().unwrap());
-        }
-        result
+        context.insert(serial_number, result.clone());
+        Ok(result)
     }
     let mut context: HashMap<usize, CircuitRc> = HashMap::default();
     deep_convert_partial_circ(
@@ -382,3 +372,53 @@ pub fn parse_compiler_repr_bijection(
         tensor_cache,
     )
 }
+
+#[apply(python_error_exception)]
+#[base_error_name(Parse)]
+#[base_exception(PyValueError)]
+#[derive(Error, Debug, Clone)]
+pub enum ParseError {
+    #[error("Parsing no regex match '{line}'")]
+    NoRegexMatch { line: String },
+
+    #[error("Parsing number failed {string}")]
+    NumberFail { string: String },
+
+    #[error("Parsing failed {string}")]
+    Fail { string: String },
+
+    #[error("Expected UUID string, found  {string}")]
+    InvalidUuid { string: String },
+
+    #[error("Parsing wrong number of children, expected {expected} found {found}")]
+    WrongNumberChildren { expected: usize, found: usize },
+
+    #[error("Parsing invalid indentation, tab width {tab_width} num spaces {spaces} stack indentation {stack_indentation} stack top {stack_top:?}")]
+    InvalidIndentation {
+        spaces: usize,
+        tab_width: usize,
+        stack_indentation: usize,
+        stack_top: Option<String>,
+    },
+
+    #[error("Parsing invalid circuit variant {v}")]
+    InvalidVariant { v: String },
+
+    #[error("Parsing invalid serial number {serial_number}")]
+    InvalidSerialNumber { serial_number: usize },
+
+    #[error("Parsing Extra Unneeded String '{string}'")]
+    ExtraUnneededString { string: String },
+
+    #[error("Parsing shape needed but not provided on {variant}")]
+    ShapeNeeded { variant: String },
+
+    #[error("Parsing found multiple circuits in sequence, instead of one circuit (aka last node wasn't child of anything)")]
+    GotMultipleCircuits {},
+
+    #[error("Circuit by name not found: {name}")]
+    CircuitRefNotFound { name: String },
+
+    #[error("Module by name not found: {name}")]
+    ModuleNotFound { name: String },
+}
