@@ -1,5 +1,6 @@
+use pyo3::once_cell::GILLazyPy;
+
 use super::CircuitRc;
-use crate::lazy::GILLazyPy;
 use crate::pyo3_prelude::*;
 
 pub struct PyCircuitItems {
