@@ -1,7 +1,8 @@
+use std::{collections::BTreeMap, fs::File, io::Write};
+
 use itertools::Itertools;
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
-use smallvec::Array;
-use std::{collections::BTreeMap, fs::File, io::Write};
+use smallvec::{Array, SmallVec as Sv};
 use uuid::Uuid;
 
 use super::{OpSize, RearrangeSpec, TensorAxisIndex, TensorIndex};
@@ -9,7 +10,6 @@ use crate::{
     all_imports::TorchDeviceDtypeOp, opt_einsum::EinsumSpec, rrfs::save_tensor_rrfs,
     tensor_util::Shape,
 };
-use smallvec::SmallVec as Sv;
 
 pub trait RustRepr {
     /// Serialize objects to Rust expression source code
