@@ -1,6 +1,8 @@
-use super::{module_nodes::replace_expand_bottom_up, CircResult, CircuitRc, DiscreteVar};
+use anyhow::Result;
 
-pub fn discrete_var_sample_all<F>(circuit: CircuitRc, should_sample: F) -> CircResult
+use super::{expand_node::replace_expand_bottom_up, CircuitRc, DiscreteVar};
+
+pub fn discrete_var_sample_all<F>(circuit: CircuitRc, should_sample: F) -> Result<CircuitRc>
 where
     F: Fn(&DiscreteVar) -> bool,
 {
