@@ -1,23 +1,22 @@
-use smallvec::Array;
+use std::{hash::Hash, iter::zip};
+
+use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
+use smallvec::{Array, SmallVec as Sv};
 
 use super::{
     algebraic_rewrite::{get_removable_axes, remove_axes},
     prelude::*,
     Add, Concat, Einsum, Index, Rearrange, ScalarConstant, Scatter,
 };
-use crate::pyo3_prelude::*;
-use crate::{all_imports::Shape, util::cumsum};
 use crate::{
+    all_imports::Shape,
+    pyo3_prelude::*,
     rearrange_spec::RearrangeSpec,
     tensor_util::{
         compose, uslices_shrink_base, uslices_to_index, TensorAxisIndex, TensorIndex, USlice,
     },
-    util::filter_out_idx,
+    util::{cumsum, filter_out_idx},
 };
-use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
-use smallvec::SmallVec as Sv;
-use std::hash::Hash;
-use std::iter::zip;
 
 /// right now writing scatter_fuse, einsum_pull_scatter, add_pull_scatter
 #[pyfunction]
