@@ -1,23 +1,24 @@
 // temporary schedule sending setup
+use anyhow::Result;
 use miniserde::{json, Deserialize, Serialize};
 use pyo3::types::PyByteArray;
+use rustc_hash::FxHashMap as HashMap;
 
 use super::{
     parse_compiler_repr_bijection,
     print::repr_circuit_line_compiler,
     scheduled_execution::{get_children_keys, Instruction, Schedule},
-    CircuitConstructionError, IrreducibleNode, ScalarConstant,
+    IrreducibleNode, ScalarConstant,
 };
-use crate::circuit::CircuitNode;
-use crate::lru_cache::TensorCacheRrfs;
-use crate::{all_imports::Shape, py_types::un_flat_concat};
-use crate::{all_imports::TorchDeviceDtype, sv};
 use crate::{
-    py_types::{Tensor, PY_UTILS},
+    all_imports::{Shape, TorchDeviceDtype},
+    circuit::CircuitNode,
+    lru_cache::TensorCacheRrfs,
+    py_types::{un_flat_concat, Tensor, PY_UTILS},
     pycall,
     pyo3_prelude::*,
+    sv,
 };
-use rustc_hash::FxHashMap as HashMap;
 
 #[pyclass]
 #[derive(Serialize, Deserialize, Debug, Clone)]
@@ -80,11 +81,7 @@ impl ScheduleToSend {
 }
 
 impl ScheduleToSend {
-    pub fn load(
-        self,
-        device: String,
-        cache: &mut Option<TensorCacheRrfs>,
-    ) -> Result<Schedule, CircuitConstructionError> {
+    pub fn load(self, device: String, cache: &mut Option<TensorCacheRrfs>) -> Result<Schedule> {
         let mut result = Schedule {
             instructions: vec![],
             constants: Default::default(),
@@ -116,7 +113,7 @@ impl ScheduleToSend {
                 let irreducible: Option<IrreducibleNode> = (**parsed).clone().into();
                 Ok((*k, irreducible.unwrap()))
             })
-            .collect::<Result<_, CircuitConstructionError>>()?;
+            .collect::<Result<_>>()?;
         result.scalar_constants = self
             .scalar_constants
             .iter()
@@ -137,7 +134,7 @@ impl ScheduleToSend {
                     resulty
                 }))
             })
-            .collect::<Result<_, CircuitConstructionError>>()?;
+            .collect::<Result<_>>()?;
         result.instructions = self
             .instructions
             .iter()
@@ -167,7 +164,7 @@ impl ScheduleToSend {
                     }
                 }
             })
-            .collect::<Result<Vec<Instruction>, CircuitConstructionError>>()?;
+            .collect::<Result<Vec<Instruction>>>()?;
         Ok(result)
     }
 }
