@@ -1,5 +1,14 @@
 use std::fmt::{self, Display};
 
+use anyhow::{bail, Context, Result};
+use itertools::Itertools;
+use macro_rules_attribute::apply;
+use miniserde::json;
+use num_bigint::BigUint;
+use pyo3::exceptions::PyValueError;
+use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
+use thiserror::Error;
+
 use super::{
     circuit_optimizer::{OptimizationContext, OptimizationSettings},
     circuit_utils::{hash_to_node, toposort_circuit},
@@ -11,26 +20,18 @@ use super::{
     visit_circuit_unwrap, Add, ArrayConstant, HashBytes, IrreducibleNode, Leaf, ModuleNodeSpec,
     ScalarConstant, Symbol,
 };
-use crate::lru_cache::TensorCacheRrfs;
-use crate::py_types::{scalar_to_tensor, tensor_scale, un_flat_concat, PY_UTILS};
-use crate::pyo3_prelude::*;
-use crate::util::filter_to_idx;
 use crate::{
     cached_lambda,
-    py_types::{ExtraPySelfOps, Tensor},
+    circuit::schedule_send::ScheduleToSend,
+    lru_cache::TensorCacheRrfs,
+    py_types::{scalar_to_tensor, tensor_scale, un_flat_concat, ExtraPySelfOps, Tensor, PY_UTILS},
     pycall,
+    pyo3_prelude::*,
+    python_error_exception,
     tensor_util::{Shape, TorchDeviceDtype},
     timed,
+    util::{arc_ref_clone, filter_to_idx},
 };
-use crate::{circuit::schedule_send::ScheduleToSend, util::arc_ref_clone};
-use itertools::Itertools;
-use macro_rules_attribute::apply;
-use miniserde::json;
-use num_bigint::BigUint;
-use pyo3::{create_exception, exceptions::PyException};
-use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
-
-use thiserror::Error;
 
 #[derive(Clone, Debug)]
 pub enum Instruction {
@@ -197,7 +198,7 @@ impl Schedule {
         string: String,
         device: String,
         tensor_cache: Option<TensorCacheRrfs>,
-    ) -> Result<Self, CircuitConstructionError> {
+    ) -> Result<Self> {
         let mut tensor_cache = tensor_cache;
         Schedule::deserialize(string, device, &mut tensor_cache)
     }
@@ -220,9 +221,9 @@ impl Schedule {
         string: String,
         device: String,
         tensor_cache: &mut Option<TensorCacheRrfs>,
-    ) -> Result<Self, CircuitConstructionError> {
-        let sent: ScheduleToSend = json::from_str(&string)
-            .map_err(|_e| CircuitConstructionError::ParsingFail { string })?;
+    ) -> Result<Self> {
+        let sent: ScheduleToSend =
+            json::from_str(&string).context("schedule deserialization failed due to json error")?;
         sent.load(device, tensor_cache)
     }
 }
@@ -281,33 +282,29 @@ pub struct ScheduleStats {
     max_circuit_set: HashSet<CircuitRc>,
 }
 
-#[derive(Error, Debug)]
-pub enum SchedulingError {
+#[apply(python_error_exception)]
+#[base_error_name(SchedulingOOM)]
+#[base_exception(PyValueError)]
+#[derive(Error, Clone, Debug)]
+pub enum SchedulingOOMError {
     #[error("{string}")]
-    CannotFitInMemory {
+    Many {
         max_memory: usize,
         memory_chunks: usize,
         node_memories: Vec<usize>,
         string: String,
     },
     #[error("Single element doesn't fit {numel}")]
-    SingleElementTooBig { numel: BigUint },
-
-    #[error("Circuit has leaf symbols")]
-    LeafSymbols {},
+    Single { numel: BigUint },
 }
 
-create_exception!(
-    rust_circuit,
-    PyOOMError,
-    PyException,
-    "OOM error from compiler"
-);
-
-impl From<SchedulingError> for PyErr {
-    fn from(err: SchedulingError) -> Self {
-        PyErr::new::<PyOOMError, _>(format!("error (TODO: better) {}", err))
-    }
+#[apply(python_error_exception)]
+#[base_error_name(Scheduling)]
+#[base_exception(PyValueError)]
+#[derive(Error, Debug)]
+pub enum SchedulingError {
+    #[error("Circuit has leaf symbols")]
+    LeafSymbols {},
 }
 
 impl Display for ScheduleStats {
@@ -850,15 +847,15 @@ fn subset_order_to_full_order(
 pub fn circuit_to_schedule(
     circuit: CircuitRc,
     context: &mut OptimizationContext,
-) -> Result<Schedule, SchedulingError> {
+) -> Result<Schedule> {
     let dag = circuits_to_dag(&[circuit.clone()]);
     if circuit.info().max_non_input_size > BigUint::from(context.cache.fallback_total_numel) {
-        return Err(SchedulingError::SingleElementTooBig {
+        bail!(SchedulingOOMError::Single {
             numel: circuit.info().max_non_input_size.clone(),
         });
     }
     let order_result = {
-        let mut result = Err(SchedulingError::CannotFitInMemory {
+        let mut result = Err(SchedulingOOMError::Many {
             max_memory: 0,
             memory_chunks: 0,
             node_memories: vec![],
@@ -946,10 +943,7 @@ pub fn circuit_to_schedule_naive_toposort(circuit: CircuitRc) -> Schedule {
 }
 
 #[pyfunction]
-pub fn scheduled_evaluate(
-    circuit: CircuitRc,
-    settings: OptimizationSettings,
-) -> Result<Tensor, SchedulingError> {
+pub fn scheduled_evaluate(circuit: CircuitRc, settings: OptimizationSettings) -> Result<Tensor> {
     let schedule = circuit_to_schedule(
         circuit.clone(),
         &mut OptimizationContext::new_settings_circuit(settings, circuit),
