@@ -1,20 +1,20 @@
 use std::collections::BTreeMap;
 
+use anyhow::{bail, Result};
+use macro_rules_attribute::apply;
+use uuid::{uuid, Uuid};
+
 use super::{
-    CachedCircuitInfo, CircuitConstructionError, CircuitNode, CircuitNodeAutoName, CircuitRc,
-    ScalarConstant, TensorEvalError,
+    CachedCircuitInfo, CircuitNode, CircuitNodeAutoName, CircuitRc, ConstructError, ScalarConstant,
+    TensorEvalError,
 };
-use crate::new_rc_unwrap;
 use crate::{
     circuit::PyCircuitBase,
-    circuit_node_auto_impl, circuit_node_extra_impl, new_rc,
+    circuit_node_auto_impl, circuit_node_extra_impl, new_rc, new_rc_unwrap,
     py_types::Tensor,
     pyo3_prelude::*,
     tensor_util::{Shape, TorchDeviceDtype},
 };
-use macro_rules_attribute::apply;
-use uuid::uuid;
-use uuid::Uuid;
 
 /// Tags a Circuit with a UUID. Use this to make two otherwise equal nodes distinct.
 /// This is used to make two independent samplings of the random variable, whereas without making them distinct,
@@ -80,11 +80,10 @@ impl CircuitNode for AutoTag {
         Box::new(std::iter::once(self.node.clone()))
     }
 
-    fn map_children_enumerate<F, E>(&self, mut f: F) -> Result<Self, CircuitConstructionError>
+    fn map_children_enumerate<F>(&self, mut f: F) -> Result<Self>
     where
         Self: Sized,
-        CircuitConstructionError: From<E>,
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc, E>,
+        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
     {
         Ok(Self::new(
             f(0, self.node.clone())?,
@@ -93,11 +92,7 @@ impl CircuitNode for AutoTag {
         ))
     }
 
-    fn eval_tensors(
-        &self,
-        tensors: &[Tensor],
-        device_dtype: &TorchDeviceDtype,
-    ) -> Result<Tensor, TensorEvalError> {
+    fn eval_tensors(&self, tensors: &[Tensor], device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
         self.node.eval_tensors(tensors, device_dtype)
     }
 }
@@ -139,17 +134,17 @@ impl DiscreteVar {
         values: CircuitRc,
         probs_and_group: CircuitRc,
         name: Option<String>,
-    ) -> (Result<Self, CircuitConstructionError>) {
+    ) -> Result<Self> {
         if probs_and_group.info().rank() != 1 {
-            return Err(CircuitConstructionError::DiscreteVarProbsMustBe1d {
+            bail!(ConstructError::DiscreteVarProbsMustBe1d {
                 shape: probs_and_group.info().shape.clone(),
             });
         }
         if values.info().rank() < 1 {
-            return Err(CircuitConstructionError::DiscreteVarNoSamplesDim {});
+            bail!(ConstructError::DiscreteVarNoSamplesDim {});
         }
         if values.info().shape[0] != probs_and_group.info().shape[0] {
-            return Err(CircuitConstructionError::DiscreteVarWrongSamplesDim {
+            bail!(ConstructError::DiscreteVarWrongSamplesDim {
                 node: values.info().shape[0],
                 probs: probs_and_group.info().shape[0],
             });
@@ -222,11 +217,10 @@ impl CircuitNode for DiscreteVar {
         Box::new([self.values.clone(), self.probs_and_group.clone()].into_iter())
     }
 
-    fn map_children_enumerate<F, E>(&self, mut f: F) -> Result<Self, CircuitConstructionError>
+    fn map_children_enumerate<F>(&self, mut f: F) -> Result<Self>
     where
         Self: Sized,
-        CircuitConstructionError: From<E>,
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc, E>,
+        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
     {
         Self::try_new(
             f(0, self.values.clone())?,
@@ -239,10 +233,11 @@ impl CircuitNode for DiscreteVar {
         &self,
         _tensors: &[Tensor],
         _device_dtype: &TorchDeviceDtype,
-    ) -> Result<Tensor, TensorEvalError> {
+    ) -> Result<Tensor> {
         Err(TensorEvalError::NotExplicitlyComputable {
             circuit: self.crc(),
-        })
+        }
+        .into())
     }
 }
 
@@ -259,12 +254,9 @@ impl DiscreteVar {
         Ok(out.into_init())
     }
     #[staticmethod]
-    pub fn new_uniform(
-        values: CircuitRc,
-        name: Option<String>,
-    ) -> Result<Self, CircuitConstructionError> {
+    pub fn new_uniform(values: CircuitRc, name: Option<String>) -> Result<Self> {
         if values.info().rank() < 1 {
-            return Err(CircuitConstructionError::DiscreteVarNoSamplesDim {});
+            bail!(ConstructError::DiscreteVarNoSamplesDim {});
         }
         Self::try_new(
             values.clone(),
@@ -299,14 +291,12 @@ impl StoredCumulantVar {
         cumulants: BTreeMap<usize, CircuitRc>,
         uuid: Uuid,
         name: Option<String>,
-    ) -> (Result<Self, CircuitConstructionError>) {
+    ) -> Result<Self> {
         if !cumulants.contains_key(&1) || !cumulants.contains_key(&2) {
-            return Err(CircuitConstructionError::StoredCumulantVarNeedsMeanVariance {});
+            bail!(ConstructError::StoredCumulantVarNeedsMeanVariance {});
         }
         if cumulants.contains_key(&0) {
-            return Err(
-                CircuitConstructionError::StoredCumulantVarInvalidCumulantNumber { number: 0 },
-            );
+            bail!(ConstructError::StoredCumulantVarInvalidCumulantNumber { number: 0 },);
         }
         let shape = &cumulants[&1].info().shape;
         for (k, v) in cumulants.iter() {
@@ -317,13 +307,11 @@ impl StoredCumulantVar {
                 .copied()
                 .collect();
             if shape_here != v.info().shape {
-                return Err(
-                    CircuitConstructionError::StoredCumulantVarCumulantWrongShape {
-                        cumulant_shape: v.info().shape.clone(),
-                        cumulant_number: *k,
-                        base_shape: shape.clone(),
-                    },
-                );
+                bail!(ConstructError::StoredCumulantVarCumulantWrongShape {
+                    cumulant_shape: v.info().shape.clone(),
+                    cumulant_number: *k,
+                    base_shape: shape.clone(),
+                },);
             }
         }
         let mut out = Self {
@@ -342,7 +330,7 @@ impl StoredCumulantVar {
         higher_cumulants: BTreeMap<usize, CircuitRc>,
         uuid: Option<Uuid>,
         name: Option<String>,
-    ) -> Result<Self, CircuitConstructionError> {
+    ) -> Result<Self> {
         let mut higher_cumulants = higher_cumulants;
         higher_cumulants.insert(1, mean);
         higher_cumulants.insert(2, variance);
@@ -418,11 +406,10 @@ impl CircuitNode for StoredCumulantVar {
         Box::new(self.cumulants.values().cloned())
     }
 
-    fn map_children_enumerate<F, E>(&self, mut f: F) -> Result<Self, CircuitConstructionError>
+    fn map_children_enumerate<F>(&self, mut f: F) -> Result<Self>
     where
         Self: Sized,
-        CircuitConstructionError: From<E>,
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc, E>,
+        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
     {
         Self::try_new(
             self.cumulants
@@ -439,10 +426,11 @@ impl CircuitNode for StoredCumulantVar {
         &self,
         _tensors: &[Tensor],
         _device_dtype: &TorchDeviceDtype,
-    ) -> Result<Tensor, TensorEvalError> {
+    ) -> Result<Tensor> {
         Err(TensorEvalError::NotExplicitlyComputable {
             circuit: self.crc(),
-        })
+        }
+        .into())
     }
 }
 
@@ -468,7 +456,7 @@ impl StoredCumulantVar {
         higher_cumulants: BTreeMap<usize, CircuitRc>,
         uuid: Option<Uuid>,
         name: Option<String>,
-    ) -> Result<Self, CircuitConstructionError> {
+    ) -> Result<Self> {
         Self::new_mv(mean, variance, higher_cumulants, uuid, name)
     }
 }
