@@ -14,17 +14,12 @@ pub use uuid;
 
 #[cfg(not(feature = "real-pyo3"))]
 mod pyo3_prelude {
-    pub use fake_pyo3::noop as pymethods;
-    pub use fake_pyo3::noop as getter;
-    pub use fake_pyo3::noop as pyclass;
-    pub use fake_pyo3::noop as pyfunction;
-    pub use fake_pyo3::noop as pymodule;
-    pub use fake_pyo3::noop as new;
-    pub use fake_pyo3::noop as args;
-    pub use fake_pyo3::noop as pyo3;
-    pub use fake_pyo3::noop as staticmethod;
+    pub use fake_pyo3::{
+        noop as pymethods, noop as getter, noop as pyclass, noop as pyfunction, noop as pymodule,
+        noop as new, noop as args, noop as pyo3, noop as staticmethod,
+    };
     /// This class ignores pyo3 attributes
-    pub use fake_pyo3::FakePyClassDeriv as PyClassDeriv;
+    pub use fake_pyo3::{FakePyClassDeriv as PyClassDeriv, FakePyClassDeriv as FromPyObject};
     pub use pyo3::prelude::{
         wrap_pyfunction, FromPyObject, GILGuard, IntoPy, IntoPyPointer, Py, PyAny, PyCell,
         PyClassInitializer, PyErr, PyModule, PyObject, PyRef, PyRefMut, PyResult, PyTryFrom,
@@ -43,6 +38,8 @@ pub mod all_imports;
 mod caching;
 pub mod circuit;
 pub mod eq_by_big_hash;
+#[macro_use]
+mod error;
 pub mod lazy;
 mod lru_cache;
 pub mod opt_einsum;
@@ -57,6 +54,8 @@ pub mod tensor_util;
 mod union_find;
 pub mod util;
 
+pub use error::print_exception_stubs;
+
 #[macro_export]
 macro_rules! sv {
     [$($tt:tt)*] => {
@@ -70,42 +69,47 @@ macro_rules! sv {
 #[cfg(feature = "real-pyo3")]
 #[pymodule]
 fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
-    use circuit::algebraic_rewrite::{
-        add_collapse_scalar_inputs, add_deduplicate, add_elim_zeros, add_flatten_once,
-        add_fuse_scalar_multiples, add_make_broadcasts_explicit, add_pull_removable_axes,
-        concat_elim_identity, concat_merge_uniform, concat_pull_removable_axes,
-        concat_repeat_to_rearrange, distribute, distribute_all, einsum_concat_to_add,
-        einsum_elim_identity, einsum_elim_zero, einsum_flatten_once, einsum_merge_scalars,
-        einsum_nest_path, einsum_of_permute_merge, einsum_pull_removable_axes, extract_add,
-        generalfunction_pull_removable_axes, index_elim_identity, index_fuse, index_merge_scalar,
-        index_split_axes, permute_of_einsum_merge, push_down_index, rearrange_elim_identity,
-        rearrange_fuse, rearrange_merge_scalar, remove_add_few_input,
+    pyo3::anyhow::set_get_exception(std::sync::Arc::new(error::get_exception));
+
+    use circuit::{
+        algebraic_rewrite::{
+            add_collapse_scalar_inputs, add_deduplicate, add_elim_zeros, add_flatten_once,
+            add_fuse_scalar_multiples, add_make_broadcasts_explicit, add_pull_removable_axes,
+            concat_elim_identity, concat_merge_uniform, concat_pull_removable_axes,
+            concat_repeat_to_rearrange, distribute, distribute_all, einsum_concat_to_add,
+            einsum_elim_identity, einsum_elim_zero, einsum_flatten_once, einsum_merge_scalars,
+            einsum_nest_path, einsum_of_permute_merge, einsum_pull_removable_axes, extract_add,
+            generalfunction_pull_removable_axes, index_elim_identity, index_fuse,
+            index_merge_scalar, index_split_axes, permute_of_einsum_merge, push_down_index,
+            rearrange_elim_identity, rearrange_fuse, rearrange_merge_scalar, remove_add_few_input,
+        },
+        canonicalize::{canonicalize_node_py, deep_canonicalize_py},
+        circuit_manipulation::{
+            filter_nodes_py, path_get, replace_nodes_py, update_nodes_py, update_path_py,
+        },
+        circuit_optimizer::{optimize_and_evaluate, optimize_and_evaluate_many},
+        circuit_utils::{
+            cast_circuit, count_nodes, toposort_circuit, total_arrayconstant_size, total_flops,
+        },
+        compiler_heuristics::deep_maybe_distribute_py,
+        concat_rewrite::{
+            add_pull_concat, concat_drop_size_zero, concat_fuse, einsum_pull_concat,
+            generalfunction_pull_concat, index_concat_drop_unreached, split_to_concat,
+        },
+        deep_rewrite::{
+            compiler_simp_py, compiler_simp_step_py, compiler_simp_until_same,
+            deep_heuristic_nest_adds, deep_pull_concat, deep_pull_concat_messy,
+            deep_push_down_index,
+        },
+        diag_rewrite::{add_pull_diags, einsum_push_down_trace},
+        flat_concat,
+        named_axes::{propagate_named_axes, set_named_axes_py},
+        scatter_rewrite::{
+            add_pull_scatter, einsum_pull_scatter, index_einsum_to_scatter, scatter_elim_identity,
+            scatter_pull_removable_axes, scatter_to_concat,
+        },
+        scheduled_execution::scheduled_evaluate,
     };
-    use circuit::canonicalize::{canonicalize_node_py, deep_canonicalize_py};
-    use circuit::circuit_manipulation::{
-        filter_nodes_py, path_get, replace_nodes_py, update_nodes_py, update_path_py,
-    };
-    use circuit::circuit_optimizer::{optimize_and_evaluate, optimize_and_evaluate_many};
-    use circuit::circuit_utils::{
-        cast_circuit, count_nodes, toposort_circuit, total_arrayconstant_size, total_flops,
-    };
-    use circuit::compiler_heuristics::deep_maybe_distribute_py;
-    use circuit::concat_rewrite::{
-        add_pull_concat, concat_drop_size_zero, concat_fuse, einsum_pull_concat,
-        generalfunction_pull_concat, index_concat_drop_unreached, split_to_concat,
-    };
-    use circuit::deep_rewrite::{
-        compiler_simp_py, compiler_simp_step_py, compiler_simp_until_same,
-        deep_heuristic_nest_adds, deep_pull_concat, deep_pull_concat_messy, deep_push_down_index,
-    };
-    use circuit::diag_rewrite::{add_pull_diags, einsum_push_down_trace};
-    use circuit::flat_concat;
-    use circuit::named_axes::{propagate_named_axes, set_named_axes_py};
-    use circuit::scatter_rewrite::{
-        add_pull_scatter, einsum_pull_scatter, index_einsum_to_scatter, scatter_elim_identity,
-        scatter_pull_removable_axes, scatter_to_concat,
-    };
-    use circuit::scheduled_execution::{scheduled_evaluate, PyOOMError};
     use opt_einsum::optimize_einsum_spec_cached;
     use pyo3::exceptions::PyValueError;
 
@@ -132,7 +136,6 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     m.add_class::<circuit::Scatter>()?;
     m.add_class::<circuit::circuit_optimizer::OptimizationSettings>()?;
     m.add_class::<circuit::circuit_optimizer::OptimizationContext>()?;
-    m.add("PyOOMError", py.get_type::<PyOOMError>())?;
 
     m.add_class::<tensor_util::TorchDeviceDtype>()?;
     m.add_class::<tensor_util::TorchDeviceDtypeOp>()?;
@@ -316,7 +319,7 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     m.add_function(wrap_pyfunction!(update_nodes_py, m)?)?;
     m.add_function(wrap_pyfunction!(path_get, m)?)?;
     m.add_function(wrap_pyfunction!(update_path_py, m)?)?;
-    m.add_function(wrap_pyfunction!(circuit::module_nodes::expand_node_py, m)?)?;
+    m.add_function(wrap_pyfunction!(circuit::expand_node::expand_node_py, m)?)?;
     m.add_function(wrap_pyfunction!(rrfs::save_tensor_rrfs, m)?)?;
     m.add_function(wrap_pyfunction!(rrfs::tensor_from_hash, m)?)?;
     m.add_function(wrap_pyfunction!(
@@ -338,11 +341,11 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
         m
     )?)?;
     m.add_function(wrap_pyfunction!(
-        circuit::module_nodes::replace_expand_bottom_up_dict_py,
+        circuit::expand_node::replace_expand_bottom_up_dict_py,
         m
     )?)?;
     m.add_function(wrap_pyfunction!(
-        circuit::module_nodes::replace_expand_bottom_up_py,
+        circuit::expand_node::replace_expand_bottom_up_py,
         m
     )?)?;
 
@@ -355,5 +358,8 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     )?)?;
 
     m.add_function(wrap_pyfunction!(circuit::print_circuit_type_check, m)?)?;
+
+    error::register_exceptions(py, m)?;
+
     Ok(())
 }
