@@ -1,4 +1,17 @@
-use crate::circuit::{CircuitConstructionError, Einsum, EinsumAxes, Symbol};
+use std::{
+    cmp::{max, min},
+    rc::Rc,
+};
+
+use cached::proc_macro::cached;
+use num_bigint::BigUint;
+use pyo3::{
+    exceptions::PyValueError,
+    types::{PyDict, PyList},
+};
+use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
+use uuid::Uuid;
+
 /// This file optimizes tensor contraction order, like Python's opt_einsum
 /// It specifies contraction order using rotating id's, like opt_einsum or numpy,
 /// which is kindof awkward but compatible
@@ -11,22 +24,10 @@ use crate::circuit::{CircuitConstructionError, Einsum, EinsumAxes, Symbol};
 /// 4: contract subgraphs together
 use crate::pyo3_prelude::*;
 use crate::{
+    circuit::{ConstructError, Einsum, EinsumAxes, Symbol},
     union_find::UnionFind,
     util::{dict_to_list, ALPHABET, ALPHABET_INV},
 };
-use cached::proc_macro::cached;
-use num_bigint::BigUint;
-use pyo3::{
-    exceptions::PyValueError,
-    types::{PyDict, PyList},
-};
-use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
-use uuid::Uuid;
-
-use std::{
-    cmp::{max, min},
-    rc::Rc,
-};
 
 /// returns vec of vecs of indices of operands
 pub fn get_disconnected_sugraphs(
@@ -328,7 +329,7 @@ impl EinsumSpec {
                     ALPHABET_INV
                         .get(&x.to_string())
                         .cloned()
-                        .ok_or(CircuitConstructionError::EinsumStringInvalid {
+                        .ok_or(ConstructError::EinsumStringInvalid {
                             string: string.clone(),
                             substring: x.to_string(),
                         })
@@ -345,7 +346,7 @@ impl EinsumSpec {
                             ALPHABET_INV
                                 .get(&c.to_string())
                                 .cloned()
-                                .ok_or(CircuitConstructionError::EinsumStringInvalid {
+                                .ok_or(ConstructError::EinsumStringInvalid {
                                     string: string.to_owned(),
                                     substring: c.to_string(),
                                 })
