@@ -1,12 +1,15 @@
-use crate::{all_imports::TorchDeviceDtype, pyo3_prelude::*};
+use anyhow::{Context, Result};
 use macro_rules_attribute::apply;
-use pyo3::pyclass::CompareOp;
-use pyo3::types::IntoPyDict;
-use pyo3::types::PyTuple;
+use pyo3::{
+    once_cell::GILLazyPy,
+    pyclass::CompareOp,
+    types::{IntoPyDict, PyTuple},
+};
 
 use crate::{
+    all_imports::TorchDeviceDtype,
     circuit::{EinsumAxes, HashBytes},
-    lazy::GILLazyPy,
+    pyo3_prelude::*,
     tensor_util::Shape,
 };
 
@@ -101,11 +104,13 @@ macro_rules! make_py_func {
 
 #[apply(make_py_func)]
 #[py_ident(py)]
-pub fn scalar_to_tensor(v: f64, shape: Shape, device_dtype: TorchDeviceDtype) -> PyResult<Tensor> {
+pub fn scalar_to_tensor(v: f64, shape: Shape, device_dtype: TorchDeviceDtype) -> Result<Tensor> {
     PY_UTILS
         .scalar_to_tensor
-        .call(py, (v, shape, device_dtype), None)?
+        .call(py, (v, shape, device_dtype), None)
+        .context("scalar to tensor")?
         .extract::<Tensor>(py)
+        .context("scalar to tensor extract")
 }
 
 #[apply(make_py_func)]
@@ -136,11 +141,13 @@ pub fn einops_repeat(
     tensor: &Tensor,
     op: String,
     sizes: impl IntoIterator<Item = (String, u64)>,
-) -> PyResult<Tensor> {
+) -> Result<Tensor> {
     PY_EINOPS
         .repeat
-        .call(py, (tensor, op), Some(sizes.into_py_dict(py)))?
+        .call(py, (tensor, op), Some(sizes.into_py_dict(py)))
+        .context("einops repeat")?
         .extract(py)
+        .context("einops repeat extract")
 }
 
 #[apply(make_py_func)]
@@ -148,7 +155,8 @@ pub fn einops_repeat(
 pub fn un_flat_concat(tensor: &Tensor, split_shapes: Vec<Shape>) -> PyResult<Vec<Tensor>> {
     PY_UTILS
         .un_flat_concat
-        .call(py, (tensor, split_shapes), None)?
+        .call(py, (tensor, split_shapes), None)
+        .context("un flat concat")?
         .extract(py)
 }
 
@@ -192,12 +200,12 @@ macro_rules! pycall {
     ($f:expr,$args:expr) => {
         Python::with_gil(|py| $f.call(py, $args, None).unwrap().extract(py).unwrap())
     };
-    ($f:expr,$args:expr,$err_type:ty) => {
+    ($f:expr, $args:expr, anyhow) => {
         Python::with_gil(|py| {
             $f.call(py, $args, None)
-                .map_err(|z| <$err_type>::from(z))?
+                .map_err(|z| anyhow::Error::from(z))?
                 .extract(py)
-                .map_err(|z| <$err_type>::from(z))
+                .map_err(|z| anyhow::Error::from(z))
         })
     };
 }
