@@ -143,7 +143,7 @@ macro_rules! setup_callable {
 
         paste::paste!{
             #[derive(Clone)]
-            pub struct [<Raw $struct_name>](Arc<dyn Fn($($arg_ty,)*) -> PyResult<$func_ret> + Send + Sync>);
+            pub struct [<Raw $struct_name>](Arc<dyn Fn($($arg_ty,)*) -> anyhow::Result<$func_ret> + Send + Sync>);
 
             impl Debug for [<Raw $struct_name>] {
                 fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
@@ -154,7 +154,7 @@ macro_rules! setup_callable {
 
         #[pymethods]
         impl $struct_name {
-            fn __call__(&self, _py: Python<'_>, $($arg_name : $arg_ty,)*) -> PyResult<$func_ret> {
+            fn __call__(&self, _py: Python<'_>, $($arg_name : $arg_ty,)*) -> anyhow::Result<$func_ret> {
                 self.$func_name($($arg_name,)*)
             }
         }
@@ -164,13 +164,13 @@ macro_rules! setup_callable {
                 Self::new_func_err(move |n| Ok(func(n)))
             }
 
-            pub fn new_func_err<F: Fn($($arg_ty,)*) -> PyResult<$func_ret> + Send + Sync + 'static>(
+            pub fn new_func_err<F: Fn($($arg_ty,)*) -> anyhow::Result<$func_ret> + Send + Sync + 'static>(
                 func: F,
             ) -> Self {
                 Self::new_func_err_arc(Arc::new(func))
             }
 
-            pub fn new_func_err_arc(func: Arc<dyn Fn($($arg_ty,)*) -> PyResult<$func_ret> + Send + Sync>) -> Self {
+            pub fn new_func_err_arc(func: Arc<dyn Fn($($arg_ty,)*) -> anyhow::Result<$func_ret> + Send + Sync>) -> Self {
                 paste::paste! {
                     $enum_name::Raw([<Raw $struct_name>](func)).into()
                 }
