@@ -1,24 +1,24 @@
-use crate::circuit::EinsumAxes;
-use crate::pyo3_prelude::*;
-use crate::{
-    circuit::py_circuit_items::PY_CIRCUIT_ITEMS,
-    circuit::HashBytes,
-    sv,
-    tensor_util::{Shape, TensorInvariantError},
-    util::{dict_to_list, filter_out_idx, is_unique, vec_map_insert, AxisInt, ALPHABET},
-};
+use std::{fmt::Debug, iter::zip};
+
+use anyhow::{bail, Result};
 use itertools::{izip, Itertools};
+use macro_rules_attribute::apply;
 use once_cell::sync::Lazy;
-use pyo3::exceptions;
-use pyo3::types::IntoPyDict;
+use pyo3::{exceptions::PyValueError, types::IntoPyDict};
 use regex::Regex;
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
 use smallvec::SmallVec as Sv;
-use std::{fmt::Debug, iter::zip};
-
 use thiserror::Error;
 use uuid::uuid;
 
+use crate::{
+    circuit::{py_circuit_items::PY_CIRCUIT_ITEMS, EinsumAxes, HashBytes},
+    pyo3_prelude::*,
+    python_error_exception, sv,
+    tensor_util::Shape,
+    util::{dict_to_list, filter_out_idx, is_unique, vec_map_insert, AxisInt, ALPHABET},
+};
+
 /// OpSize is a memory optimization of Option<usize> that stores a 63-bit int and one "is this none" bit
 #[derive(Copy, Clone, PartialEq, Eq)]
 pub struct OpSize(pub u64);
@@ -517,7 +517,7 @@ impl RearrangeSpec {
     fn __repr__(&self) -> String {
         format!("{:?}", self)
     }
-    pub fn shapes(&self) -> Result<(Shape, Shape), TensorInvariantError> {
+    pub fn shapes(&self) -> Result<(Shape, Shape)> {
         let used_ints: HashSet<u8> = self
             .input_ints
             .iter()
@@ -531,7 +531,7 @@ impl RearrangeSpec {
             .enumerate()
             .any(|(i, x)| x.is_none() && used_ints.contains(&(i as u8)))
         {
-            return Err(TensorInvariantError::RearrangeHasWildcardSizes { spec: self.clone() });
+            bail!(RearrangeSpecError::HasWildcardSizes { spec: self.clone() });
         }
         Ok((
             self.input_ints
@@ -720,10 +720,7 @@ impl RearrangeSpec {
         }
     }
 
-    pub fn fill_empty_ints(
-        &self,
-        allow_rust_invalid: bool,
-    ) -> Result<RearrangeSpec, TensorInvariantError> {
+    pub fn fill_empty_ints(&self, allow_rust_invalid: bool) -> Result<RearrangeSpec> {
         let mut next_int = self.all_ints().max().map(|i| i + 1).unwrap_or(0);
         let first_int_to_add = next_int;
         let input_ints = self
@@ -755,7 +752,7 @@ impl RearrangeSpec {
             if !output_ints.is_empty() {
                 output_ints[0].extend(first_int_to_add..int_after_input);
             } else if !allow_rust_invalid {
-                return Err(TensorInvariantError::RearrangeNotConvertable { spec: self.clone() });
+                bail!(RearrangeSpecError::NotConvertable { spec: self.clone() });
             }
         }
         let result = RearrangeSpec {
@@ -777,15 +774,11 @@ impl RearrangeSpec {
 
     #[staticmethod]
     #[pyo3(name = "fuse")]
-    pub fn fuse_py(inner: Self, outer: Self) -> PyResult<Self> {
-        RearrangeSpec::fuse(&inner, &outer).map_err(Into::into)
+    pub fn fuse_py(inner: Self, outer: Self) -> Result<Self> {
+        RearrangeSpec::fuse(&inner, &outer)
     }
     #[pyo3(name = "conform_to_input_shape")]
-    fn conform_to_input_shape_py(
-        &self,
-        shape: Shape,
-        coerce: bool,
-    ) -> Result<RearrangeSpec, TensorInvariantError> {
+    fn conform_to_input_shape_py(&self, shape: Shape, coerce: bool) -> Result<RearrangeSpec> {
         self.conform_to_input_shape(&shape, coerce)
     }
 }
@@ -817,9 +810,9 @@ impl RearrangeSpec {
         &self,
         shape: &Shape,
         coerce: bool, /* if false, only change None sizes, if true, change others if unique solution */
-    ) -> Result<RearrangeSpec, TensorInvariantError> {
+    ) -> Result<RearrangeSpec> {
         if shape.len() != self.input_ints.len() {
-            return Err(TensorInvariantError::RearrangeInputNotConformable {
+            bail!(RearrangeSpecError::InputNotConformable {
                 spec: self.clone(),
                 shape: shape.clone(),
             });
@@ -846,13 +839,11 @@ impl RearrangeSpec {
             // dbg!(&int_sizes,&non_wildcard_size,&ints,&l);
             if non_wildcard_size != *l as u64 {
                 if none_indices.len() > 1 {
-                    return Err(TensorInvariantError::RearrangeTooManyWildcardSizes {
-                        spec: self.clone(),
-                    });
+                    bail!(RearrangeSpecError::TooManyWildcardSizes { spec: self.clone() });
                 } else if coerce && ints.len() == 1 {
                     int_sizes[ints[0] as usize] = OpSize(*l as u64);
                 } else if *l as u64 % non_wildcard_size != 0 {
-                    return Err(TensorInvariantError::RearrangeInputNotConformable {
+                    bail!(RearrangeSpecError::InputNotConformable {
                         spec: self.clone(),
                         shape: shape.clone(),
                     });
@@ -860,7 +851,7 @@ impl RearrangeSpec {
                     int_sizes[ints[none_indices[0]] as usize] =
                         OpSize(*l as u64 / non_wildcard_size);
                 } else {
-                    return Err(TensorInvariantError::RearrangeInputNotConformable {
+                    bail!(RearrangeSpecError::InputNotConformable {
                         spec: self.clone(),
                         shape: shape.clone(),
                     });
@@ -974,7 +965,7 @@ impl RearrangeSpec {
     /// );
     /// assert_eq!(RearrangeSpec::fuse(&inner, &outer).unwrap(), fused);
     /// ```
-    pub fn fuse(inner: &Self, outer: &Self) -> Result<Self, RewriteError> {
+    pub fn fuse(inner: &Self, outer: &Self) -> Result<Self> {
         let mut inner_sizes: Vec<u64> = inner.get_unwrapped_sizes();
         let mut outer_sizes: Vec<u64> = outer.get_unwrapped_sizes();
         let mut inner_replaces: HashMap<AxisInt, Vec<AxisInt>> = HashMap::default();
@@ -1037,7 +1028,7 @@ impl RearrangeSpec {
                     };
 
                     if big_s % small_s != 0 {
-                        return Err(RewriteError::RearrangeNotFusable { inner:inner.clone(), outer:outer.clone() });
+                        bail!(RearrangeSpecError::NotFusable { inner:inner.clone(), outer:outer.clone() });
                     }
                     if in_s > out_s {
                         in_to_out.insert(new_i, out_i);
@@ -1056,7 +1047,7 @@ impl RearrangeSpec {
             if inner_iter.any(|&i| inner_sizes[i as usize] != 1)
                 || outer_iter.any(|&i| outer_sizes[i as usize] != 1)
             {
-                return Err(RewriteError::RearrangeNotFusable {
+                bail!(RearrangeSpecError::NotFusable {
                     inner: inner.clone(),
                     outer: outer.clone(),
                 });
@@ -1107,17 +1098,26 @@ impl RearrangeSpec {
     }
 }
 
-#[derive(Debug, Error)]
-pub enum RewriteError {
+#[apply(python_error_exception)]
+#[base_error_name(RearrangeSpec)]
+#[base_exception(PyValueError)]
+#[derive(Error, Debug, Clone)]
+pub enum RearrangeSpecError {
+    #[error("RearrangeSpec not conformable, {spec:?} {shape:?}")]
+    InputNotConformable { spec: RearrangeSpec, shape: Shape },
+
+    #[error("RearrangeSpec has wildcard sizes, {spec:?}=")]
+    HasWildcardSizes { spec: RearrangeSpec },
+
+    #[error("RearrangeSpec cannot be converted to no-(), no-squeeze format, {spec:?}=")]
+    NotConvertable { spec: RearrangeSpec },
+
+    #[error("RearrangeSpec has too many wildcard sizes, {spec:?}")]
+    TooManyWildcardSizes { spec: RearrangeSpec },
+
     #[error("Rearranges can't be fused, {inner:?}\n{outer:?}")]
-    RearrangeNotFusable {
+    NotFusable {
         inner: RearrangeSpec,
         outer: RearrangeSpec,
     },
 }
-
-impl From<RewriteError> for PyErr {
-    fn from(err: RewriteError) -> Self {
-        PyErr::new::<exceptions::PyValueError, _>(format!("error {}", err))
-    }
-}
