@@ -1,12 +1,14 @@
 use std::os::unix::prelude::OsStrExt;
 
-use crate::lru_cache::TensorCacheRrfs;
-use crate::pyo3_prelude::*;
+use anyhow::{bail, Context, Result};
+use base16::encode_lower;
+
 use crate::{
     circuit::ArrayConstant,
+    lru_cache::TensorCacheRrfs,
     py_types::{Tensor, PY_UTILS},
+    pyo3_prelude::*,
 };
-use base16::encode_lower;
 
 pub fn get_rrfs_dir() -> String {
     std::env::var("RRFS_DIR").unwrap_or_else(|_e| std::env::var("HOME").unwrap() + "/rrfs")
@@ -18,31 +20,30 @@ pub fn get_tensor_by_hash_dir() -> String {
 }
 
 #[pyfunction]
-pub fn tensor_from_hash(hash_base16: &str) -> Result<Tensor, PyErr> {
+pub fn tensor_from_hash(hash_base16: &str) -> Result<Tensor> {
     let hashdir = get_tensor_by_hash_dir() + "/" + hash_base16 + ".pt";
-    Python::with_gil(|py| {
+    let mut t: Tensor = Python::with_gil(|py| {
         PY_UTILS
             .torch
             .getattr(py, "load")
             .unwrap()
             .call(py, (hashdir,), None)
-            .map(|z| z.extract(py).unwrap())
-            .map(|t: Tensor| {
-                if std::env::var("TENSORS_BY_HASH_REHASH_ON_LOAD").is_err() {
-                    let mut t = t;
-                    t.set_hash(Some(
-                        ::base16::decode(hash_base16).unwrap().try_into().unwrap(),
-                    ));
-                    t
-                } else {
-                    t
-                }
-            })
-    })
+            .context("Failed to load tensor from hash")?
+            .extract(py)
+            .context("Failed to extract pyobject in tensor from hash")
+    })?;
+
+    if std::env::var("TENSORS_BY_HASH_REHASH_ON_LOAD").is_err() {
+        t.set_hash(Some(
+            ::base16::decode(hash_base16).unwrap().try_into().unwrap(),
+        ));
+    }
+
+    Ok(t)
 }
 
 #[pyfunction]
-pub fn tensor_from_hash_prefix(hash_base16: &str) -> Result<Tensor, PyErr> {
+pub fn tensor_from_hash_prefix(hash_base16: &str) -> Result<Tensor> {
     let hash_base16_bytes = hash_base16.as_bytes();
     let dir: Vec<_> = std::fs::read_dir(get_tensor_by_hash_dir())
         .unwrap()
@@ -55,15 +56,10 @@ pub fn tensor_from_hash_prefix(hash_base16: &str) -> Result<Tensor, PyErr> {
         })
         .collect();
     if dir.len() > 1 {
-        return Err(PyErr::new::<pyo3::exceptions::PyTypeError, _>(
-            "prefix ambiguous",
-        ));
+        bail!("tensor hash prefix ambiguous");
     }
     if dir.is_empty() {
-        return Err(PyErr::new::<pyo3::exceptions::PyTypeError, _>(format!(
-            "prefix not found {}",
-            hash_base16
-        )));
+        bail!("tensor from hash prefix not found {}", hash_base16);
     }
     tensor_from_hash(
         dir[0]
@@ -77,10 +73,7 @@ pub fn tensor_from_hash_prefix(hash_base16: &str) -> Result<Tensor, PyErr> {
     )
 }
 
-pub fn arrayconstant_from_hash(
-    name: Option<String>,
-    hash_base16: &str,
-) -> Result<ArrayConstant, PyErr> {
+pub fn arrayconstant_from_hash(name: Option<String>, hash_base16: &str) -> Result<ArrayConstant> {
     tensor_from_hash(hash_base16).map(|value| ArrayConstant::new(value, name))
 }
 
@@ -88,7 +81,7 @@ pub fn arrayconstant_from_hash_prefix(
     name: Option<String>,
     hash_base16: &str,
     tensor_cache: &mut Option<TensorCacheRrfs>,
-) -> Result<ArrayConstant, PyErr> {
+) -> Result<ArrayConstant> {
     if let Some(tc) = tensor_cache {
         return tc
             .get_tensor(hash_base16.to_owned())
@@ -98,7 +91,7 @@ pub fn arrayconstant_from_hash_prefix(
 }
 
 #[pyfunction]
-pub fn save_tensor_rrfs(tensor: Tensor) -> Result<String, PyErr> {
+pub fn save_tensor_rrfs(tensor: Tensor) -> Result<String> {
     let tensor = tensor.hashed();
     let hash_base16 = encode_lower(tensor.hash().unwrap());
     let hashdir = get_tensor_by_hash_dir() + "/" + &hash_base16 + ".pt";
@@ -106,8 +99,9 @@ pub fn save_tensor_rrfs(tensor: Tensor) -> Result<String, PyErr> {
         PY_UTILS
             .torch
             .getattr(py, "save")
-            .unwrap()
+            .context("save tensor get save attribute")?
             .call(py, (tensor.tensor(), hashdir), None)
+            .context("save tensor")
             .map(|_| hash_base16)
     })
 }
