@@ -1,7 +1,10 @@
-use crate::lru_cache::TensorCacheRrfs;
-use crate::{circuit::scheduled_execution::Schedule, py_types::PY_UTILS, pycall, pyo3_prelude::*};
 use tiny_http::{Response, Server};
 
+use crate::{
+    circuit::scheduled_execution::Schedule, lru_cache::TensorCacheRrfs, py_types::PY_UTILS, pycall,
+    pyo3_prelude::*,
+};
+
 #[pyfunction]
 pub fn circuit_server_serve(my_url: String, device: String, tensor_cache: Option<TensorCacheRrfs>) {
     let server = Server::http(&my_url).unwrap();
