@@ -1,60 +1,40 @@
-use crate::lru_cache::TensorCacheRrfs;
-use crate::py_types::PY_UTILS;
-use crate::rrfs::{save_tensor_rrfs, tensor_from_hash_prefix};
-use crate::util::cumsum;
-use crate::{
-    circuit::{CircuitConstructionError, HashBytes},
-    filter_by_variant,
-    py_types::Tensor,
-    pycall,
-    rearrange_spec::RearrangeSpec,
-};
-use crate::{pyo3_prelude::*, sv};
-use pyo3::types::IntoPyDict;
-use pyo3::{
-    exceptions, ffi, pyclass, pymethods,
-    types::{PySlice, PyTuple},
-    AsPyPointer, FromPyObject, IntoPy, PyAny, PyErr, PyObject, PyResult, Python, ToPyObject,
-};
-use rustc_hash::FxHashMap as HashMap;
-use smallvec::SmallVec as Sv;
 use std::{
     cmp::{max, min},
     fmt::{self, Display},
     iter::zip,
 };
 
+use anyhow::{bail, Context, Result};
+use macro_rules_attribute::apply;
+use pyo3::{
+    exceptions,
+    exceptions::PyValueError,
+    ffi, pyclass, pymethods,
+    types::{IntoPyDict, PySlice, PyTuple},
+    AsPyPointer, FromPyObject, IntoPy, PyAny, PyErr, PyObject, PyResult, Python, ToPyObject,
+};
+use rustc_hash::FxHashMap as HashMap;
+use smallvec::SmallVec as Sv;
 use thiserror::Error;
 use uuid::uuid;
 
 use super::py_types::ExtraPySelfOps;
+use crate::{
+    circuit::{self, ConstructError, HashBytes},
+    filter_by_variant,
+    lru_cache::TensorCacheRrfs,
+    py_types::{Tensor, PY_UTILS},
+    pycall,
+    pyo3_prelude::*,
+    rrfs::{save_tensor_rrfs, tensor_from_hash_prefix},
+    sv,
+    util::cumsum,
+};
 
 // make me small vec for perf
 pub type Shape = Sv<[usize; 6]>; // TODO: maybe this should be struct which preserves invariant that dim shapes >= 0?
 
-#[derive(Error, Debug, Clone)]
-pub enum TensorInvariantError {
-    #[error("Shapes aren't broadcastable, {shapes:?}")]
-    NotBroadcastable { shapes: Vec<Shape> },
-    #[error("RearrangeSpec not conformable, {spec:?} {shape:?}")]
-    RearrangeInputNotConformable { spec: RearrangeSpec, shape: Shape },
-
-    #[error("RearrangeSpec has wildcard sizes, {spec:?}=")]
-    RearrangeHasWildcardSizes { spec: RearrangeSpec },
-
-    #[error("RearrangeSpec cannot be converted to no-(), no-squeeze format, {spec:?}=")]
-    RearrangeNotConvertable { spec: RearrangeSpec },
-
-    #[error("RearrangeSpec has too many wildcard sizes, {spec:?}")]
-    RearrangeTooManyWildcardSizes { spec: RearrangeSpec },
-}
-
-impl From<TensorInvariantError> for PyErr {
-    fn from(err: TensorInvariantError) -> Self {
-        PyErr::new::<exceptions::PyValueError, _>(format!("error (TODO: better) {}", err))
-    }
-}
-pub fn broadcast_shapes(shapes: &Vec<Shape>) -> Result<Shape, TensorInvariantError> {
+pub fn broadcast_shapes(shapes: &Vec<Shape>) -> Result<Shape> {
     if shapes.is_empty() {
         return Ok(sv![]);
     }
@@ -73,7 +53,7 @@ pub fn broadcast_shapes(shapes: &Vec<Shape>) -> Result<Shape, TensorInvariantErr
                             && result[out_rank - axis_from_end] != 1
                             && result[out_rank - axis_from_end] != axis_len
                     {
-                        return Err(TensorInvariantError::NotBroadcastable {
+                        bail!(MiscInputError::NotBroadcastable {
                             shapes: shapes.to_owned(),
                         });
                     }
@@ -86,6 +66,15 @@ pub fn broadcast_shapes(shapes: &Vec<Shape>) -> Result<Shape, TensorInvariantErr
     Ok(result)
 }
 
+#[apply(python_error_exception)]
+#[base_error_name(MiscInput)]
+#[base_exception(PyValueError)]
+#[derive(Error, Debug, Clone)]
+pub enum MiscInputError {
+    #[error("Shapes aren't broadcastable, {shapes:?}")]
+    NotBroadcastable { shapes: Vec<Shape> },
+}
+
 // we could bit pack if we wanted...
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
 pub struct Slice {
@@ -480,9 +469,9 @@ impl TensorIndex {
         })
     }
 
-    pub fn validate(&self, shape: &Shape) -> Result<(), CircuitConstructionError> {
+    pub fn validate(&self, shape: &Shape) -> Result<()> {
         let get_err = |at| {
-            Err(CircuitConstructionError::IndexOutOfBounds {
+            Err(ConstructError::IndexOutOfBounds {
                 index: self.clone(),
                 shape: shape.clone(),
                 at,
@@ -615,8 +604,8 @@ impl TensorIndex {
     pub fn from_bijection_string(
         string: &str,
         tensor_cache: &mut Option<TensorCacheRrfs>,
-    ) -> Result<Self, CircuitConstructionError> {
-        let er = CircuitConstructionError::ParsingFail {
+    ) -> Result<Self> {
+        let er = circuit::ParseError::Fail {
             string: string.to_owned(),
         };
         Ok(TensorIndex(
@@ -628,8 +617,8 @@ impl TensorIndex {
                 .split(',')
                 .map(str::trim)
                 .filter(|x| !x.is_empty())
-                .map(|x| {
-                    let num_er = CircuitConstructionError::ParsingNumberFail {
+                .map(|x| -> Result<_> {
+                    let num_er = circuit::ParseError::NumberFail {
                         string: x.to_string(),
                     };
                     if let Ok(int) = x.parse::<i64>() {
@@ -639,11 +628,10 @@ impl TensorIndex {
                         return tensor_cache
                             .as_mut()
                             .map(|tc| tc.get_tensor(no_t.to_owned()))
-                            .unwrap_or_else(|| tensor_from_hash_prefix(no_t))
-                            .map(TensorAxisIndex::Tensor)
-                            .map_err(|_e| CircuitConstructionError::TensorHashNotFound {
-                                hash: no_t.to_owned(),
-                            });
+                            .unwrap_or_else(|| {
+                                tensor_from_hash_prefix(no_t).context("tensor hash from string")
+                            })
+                            .map(TensorAxisIndex::Tensor);
                     }
                     if let [start, end] = x.split(':').collect::<Vec<_>>()[..] {
                         return Ok(TensorAxisIndex::Slice(Slice {
@@ -659,9 +647,9 @@ impl TensorIndex {
                             },
                         }));
                     }
-                    Err(er.clone())
+                    bail!(er.clone())
                 })
-                .collect::<Result<Vec<_>, CircuitConstructionError>>()?,
+                .collect::<Result<Vec<_>>>()?,
         ))
     }
 }
@@ -863,11 +851,11 @@ impl TorchDeviceDtypeOp {
         dtype: None,
     };
 
-    pub fn combine(self, other: Self) -> Result<Self, CircuitConstructionError> {
+    pub fn combine(self, other: Self) -> Result<Self> {
         let out = Self {
             device: match (&self.device, &other.device) {
                 (Some(l), Some(r)) if l != r => {
-                    return Err(CircuitConstructionError::ChildrenMultipleDevices {
+                    bail!(ConstructError::ChildrenMultipleDevices {
                         a: self.device.clone(),
                         b: other.device.clone(),
                     });
@@ -876,7 +864,7 @@ impl TorchDeviceDtypeOp {
             },
             dtype: match (&self.dtype, &other.dtype) {
                 (Some(l), Some(r)) if l != r => {
-                    return Err(CircuitConstructionError::ChildrenMultipleDtypes {
+                    bail!(ConstructError::ChildrenMultipleDtypes {
                         a: self.dtype.clone(),
                         b: other.dtype.clone(),
                     });
